from ApiMethod import ApiMethod


class Photos(ApiMethod):
    def __init__(self, access_token, v, session, calls_per_second):
        super(Photos, self).__init__(access_token, v, session, calls_per_second)
        self._base_method = 'photos.'

    def confirmTag(self, owner_id: str = None, photo_id: str = None, tag_id: str = None) -> dict:
        """
        Подтверждает отметку на фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param tag_id: идентификатор отметки на фотографии.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'confirmTag'
        return self._call(method_name, params)

    def copy(self, owner_id: str = None, photo_id: str = None, access_key: str = None) -> dict:
        """
        Позволяет скопировать фотографию в альбом "Сохраненные фотографии"

        :param owner_id: идентификатор владельца фотографии
        :param photo_id: индентификатор фотографии
        :param access_key: специальный код доступа для приватных фотографий

        :return: Возвращает идентификатор созданной фотографии
        """

        params = locals()
        method_name = self._base_method + 'copy'
        return self._call(method_name, params)

    def createAlbum(self, title: str = None, group_id: str = None, description: str = None, privacy_view: str = None,
                    privacy_comment: str = None, upload_by_admins_only: str = None,
                    comments_disabled: str = None) -> dict:
        """
        Создает пустой альбом для фотографий.

        :param title: название альбома.
        :param group_id: идентификатор сообщества, в котором создаётся альбом.
        :param description: текст описания альбома.
        :param privacy_view: настройки приватности просмотра альбома в специальном формате.
        :param privacy_comment: настройки приватности комментирования альбома в специальном формате.
        :param upload_by_admins_only: кто может загружать фотографии в альбом (только для альбома сообщества).
            Возможные значения:   0 — фотографии могут добавлять все пользователи;  1 — фотографии могут добавлять
            только редакторы и администраторы.
        :param comments_disabled: отключено ли комментирование альбома (только для альбома сообщества).
            Возможные значения:   0 — комментирование включено;  1 — комментирование отключено.

        :return: После успешного выполнения возвращает объект, который содержит следующие поля:   id —
            идентификатор созданного альбома;  thumb_id — идентификатор фотографии, которая является обложкой альбома
            (0, если обложка отсутствует);  owner_id идентификатор пользователя или сообщества, которому принадлежит
            альбом;  title— название альбома;  description — описание альбома;  created — дата создания альбома в
            формате unixtime;  updated — дата обновления альбома в формате unixtime;  size — количество фотографий в
            альбоме;  privacy_view — настройки приватности для альбома в формате настроек приватности (только для
            альбома пользователя);  privacy_comment — настройки приватности для альбома в формате настроек
            приватности (только для альбома пользователя);  upload_by_admins_only — кто может загружать фотографии в
            альбом (только для альбома сообщества);  comments_disabled — отключено ли комментирование альбома (только
            для альбома сообщества);  can_upload — может ли текущий пользователь добавлять фотографии в альбом
        """

        params = locals()
        method_name = self._base_method + 'createAlbum'
        return self._call(method_name, params)

    def createComment(self, owner_id: str = None, photo_id: str = None, message: str = None, attachments: str = None,
                      from_group: str = None, reply_to_comment: str = None, sticker_id: str = None,
                      access_key: str = None, guid: str = None) -> dict:
        """
        Создает новый комментарий к фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param message: текст комментария (является обязательным, если не задан параметр attachments).
            Максимальное количество символов: 2048.
        :param attachments: список объектов, приложенных к комментарию и разделённых символом ",". Поле
            attachments представляется в формате: <type><owner_id>_<media_id>,<type><owner_id>_<media_id> <type> —
            тип медиа-вложения: photo — фотография  video — видеозапись  audio — аудиозапись  doc — документ
            <owner_id> — идентификатор владельца медиа-вложения  <media_id> — идентификатор медиа-вложения.
            Например: photo100172_166443618,photo66748_265827614 Параметр является обязательным, если не задан
            параметр message.
        :param from_group: Данный параметр учитывается, если owner_id < 0 (комментарий к фотографии группы).
            Возможные значения:   1 — комментарий будет опубликован от имени группы;  0 — комментарий будет
            опубликован от имени пользователя.  По умолчанию: 0.
        :param reply_to_comment: идентификатор комментария, в ответ на который нужно оставить текущий.
        :param sticker_id: идентификатор стикера, который нужно прикрепить к комментарию.
        :param access_key: ключ доступа.
        :param guid: уникальное значение для предотвращения повторной отправки одного и того же комментария.

        :return: После успешного выполнения возвращает идентификатор созданного комментария
        """

        params = locals()
        method_name = self._base_method + 'createComment'
        return self._call(method_name, params)

    def delete(self, owner_id: str = None, photo_id: str = None) -> dict:
        """
        Удаление фотографии на сайте.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'delete'
        return self._call(method_name, params)

    def deleteAlbum(self, album_id: str = None, group_id: str = None) -> dict:
        """
        Удаляет указанный альбом для фотографий у текущего пользователя

        :param album_id: идентификатор альбома.
        :param group_id: идентификатор сообщества, в котором размещен альбом.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'deleteAlbum'
        return self._call(method_name, params)

    def deleteComment(self, owner_id: str = None, comment_id: str = None) -> dict:
        """
        Удаляет комментарий к фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param comment_id: идентификатор комментария.

        :return: После успешного выполнения возвращает 1 (0, если комментарий не найден)
        """

        params = locals()
        method_name = self._base_method + 'deleteComment'
        return self._call(method_name, params)

    def edit(self, owner_id: str = None, photo_id: str = None, caption: str = None, latitude: str = None,
             longitude: str = None, place_str: str = None, foursquare_id: str = None, delete_place: str = None) -> dict:
        """
        Редактирует описание или геометку у фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param caption: новый текст описания к фотографии. Если параметр не задан, то считается, что он равен
            пустой строке.
        :param latitude: географическая широта.
        :param longitude: географическая долгота.
        :param place_str: название места.
        :param foursquare_id: id в Foursquare.
        :param delete_place: удалить место (0 — не удалять, 1 — удалить).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'edit'
        return self._call(method_name, params)

    def editAlbum(self, album_id: str = None, title: str = None, description: str = None, owner_id: str = None,
                  privacy_view: str = None, privacy_comment: str = None, upload_by_admins_only: str = None,
                  comments_disabled: str = None) -> dict:
        """
        Редактирует данные альбома для фотографий.

        :param album_id: идентификатор альбома.
        :param title: новое название альбома.
        :param description: новый текст описания альбома.
        :param owner_id: идентификатор владельца альбома (пользователь или сообщество). Обратите внимание,
            идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например, owner_id=-1
            соответствует идентификатору сообщества ВКонтакте API (club1)
        :param privacy_view: настройки приватности просмотра альбома в специальном формате.
        :param privacy_comment: настройки приватности комментирования альбома в специальном формате.
        :param upload_by_admins_only: кто может загружать фотографии в альбом (только для альбома сообщества).
            Возможные значения:   0 — фотографии могут добавлять все пользователи;  1 — фотографии могут добавлять
            только редакторы и администраторы.
        :param comments_disabled: отключено ли комментирование альбома (только для альбома сообщества).
            Возможные значения:   0 — комментирование включено;  1 — комментирование отключено.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'editAlbum'
        return self._call(method_name, params)

    def editComment(self, owner_id: str = None, comment_id: str = None, message: str = None,
                    attachments: str = None) -> dict:
        """
        Изменяет текст комментария к фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param comment_id: идентификатор комментария.
        :param message: новый текст комментария. Обязательный параметр, если не задан параметр attachments.
            Максимальное количество символов: 2048.
        :param attachments: новый список объектов, приложенных к комментарию и разделённых символом ",". Поле
            attachments представляется в формате: <type><owner_id>_<media_id>,<type><owner_id>_<media_id> <type> —
            тип медиа-вложения: photo — фотография  video — видеозапись  audio — аудиозапись  doc — документ
            <owner_id> — идентификатор владельца медиа-вложения  <media_id> — идентификатор медиа-вложения.
            Например: photo100172_166443618,photo66748_265827614 Параметр является обязательным, если не задан
            параметр message.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'editComment'
        return self._call(method_name, params)

    def get(self, owner_id: str = None, album_id: str = None, photo_ids: str = None, rev: str = None,
            extended: str = None, feed_type: str = None, feed: str = None, photo_sizes: str = None, offset: str = None,
            count: str = None) -> dict:
        """
        Возвращает список фотографий в альбоме.

        :param owner_id: идентификатор владельца альбома. Обратите внимание, идентификатор сообщества в параметре
            owner_id необходимо указывать со знаком "-" — например, owner_id=-1 соответствует идентификатору
            сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома. Для служебных альбомов используются следующие идентификаторы:
            wall — фотографии со стены;  profile — фотографии профиля;  saved — сохраненные фотографии. Возвращается
            только с ключом доступа пользователя.
        :param photo_ids: идентификаторы фотографий, информацию о которых необходимо вернуть.
        :param rev: порядок сортировки фотографий. Возможные значения:   1 — антихронологический;  0 —
            хронологический.
        :param extended: 1 — будут возвращены дополнительные поля likes, comments, tags, can_comment, reposts. По
            умолчанию: 0.
        :param feed_type: тип новости, получаемый в поле type метода newsfeed.get, для получения только
            загруженных пользователем фотографий, либо только фотографий, на которых он был отмечен. Может принимать
            значения photo, photo_tag.
        :param feed: время в формате, который может быть получен методом newsfeed.get в поле date, для получения
            всех фотографий загруженных пользователем в определённый день либо на которых пользователь был отмечен.
            Также нужно указать параметр uid пользователя, с которым произошло событие.  Значение должно отличаться
            от текущего времени не более, чем на месяц.
        :param photo_sizes: 1 — возвращать доступные размеры фотографии в специальном формате. По умолчанию: 0.
        :param offset: отступ, необходимый для получения определенного подмножества записей.
        :param count: количество записей, которое будет получено.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов фотографий в поле items.  Если к фотографии прикреплено местоположение, также возвращаются поля
            lat и long, содержащие географические координаты отметки.  Если был задан параметр extended=1,
            возвращаются дополнительные поля:   likes — количество отметок Мне нравится и информация о том, поставил
            ли лайк текущий пользователь;  comments — количество комментариев к фотографии;  tags — количество
            отметок на фотографии;  can_comment — может ли текущий пользователь комментировать фото (1 — может, 0 —
            не может);  reposts — число репостов фотографии.   Если был задан параметр photo_sizes=1, вместо полей
            width и height возвращаются размеры копий фотографии в специальном формате
        """

        params = locals()
        method_name = self._base_method + 'get'
        return self._call(method_name, params)

    def getAlbums(self, owner_id: str = None, album_ids: str = None, offset: str = None, count: str = None,
                  need_system: str = None, need_covers: str = None, photo_sizes: str = None) -> dict:
        """
        Возвращает список фотоальбомов пользователя или сообщества.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежат альбомы. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param album_ids: перечисленные через запятую идентификаторы альбомов.
        :param offset: смещение, необходимое для выборки определенного подмножества альбомов.
        :param count: количество альбомов, которое нужно вернуть. (по умолчанию возвращаются все альбомы)
        :param need_system: 1 — будут возвращены системные альбомы, имеющие отрицательные идентификаторы.
            Обратите внимание, что информация о системных альбомах возвращается даже в том случае, если они не
            содержат фотографий.
        :param need_covers: 1 — будет возвращено дополнительное поле thumb_src с адресом изображения-обложки. По
            умолчанию поле thumb_src не возвращается.
        :param photo_sizes: 1 — размеры фотографий будут возвращены в специальном формате.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов, описывающих альбомы, в поле items. Каждый из этих объектов содержит следующие поля:   id —
            идентификатор альбома;  thumb_id — идентификатор фотографии, которая является обложкой (0, если обложка
            отсутствует);  owner_id — идентификатор владельца альбома;  title — название альбома;  description —
            описание альбома; (не приходит для системных альбомов)  created — дата создания альбома в формате
            unixtime; (не приходит для системных альбомов);  updated — дата последнего обновления альбома в формате
            unixtime; (не приходит для системных альбомов);  size — количество фотографий в альбоме;  can_upload — 1,
            если текущий пользователь может загружать фотографии в альбом (при запросе информации об альбомах
            сообщества);  privacy_view* — настройки приватности для альбома в формате настроек приватности (только
            для альбома пользователя, не приходит для системных альбомов);  privacy_comment* — настройки приватности
            для альбома в формате настроек приватности (только для альбома пользователя, не приходит для системных
            альбомов);  upload_by_admins_only* — кто может загружать фотографии в альбом (только для альбома
            сообщества, не приходит для системных альбомов);  comments_disabled* — отключено ли комментирование
            альбома (только для альбома сообщества, не приходит для системных альбомов);  thumb_src — ссылка на
            изображение обложки альбома (если был указан параметр need_covers).   * — поля возвращаются только при
            запросе информации об альбомах текущего пользователя или альбомах администрируемых им сообществ
        """

        params = locals()
        method_name = self._base_method + 'getAlbums'
        return self._call(method_name, params)

    def getAlbumsCount(self, user_id: str = None, group_id: str = None) -> dict:
        """
        Возвращает количество доступных альбомов пользователя или сообщества.

        :param user_id: идентификатор пользователя, количество альбомов которого необходимо получить.
        :param group_id: идентификатор сообщества, количество альбомов которого необходимо получить.

        :return: После успешного выполнения возвращает количество альбомов с учетом настроек приватности
        """

        params = locals()
        method_name = self._base_method + 'getAlbumsCount'
        return self._call(method_name, params)

    def getAll(self, owner_id: str = None, extended: str = None, offset: str = None, count: str = None,
               photo_sizes: str = None, no_service_albums: str = None, need_hidden: str = None,
               skip_hidden: str = None) -> dict:
        """
        Возвращает все фотографии пользователя или сообщества в антихронологическом порядке.

        :param owner_id: идентификатор пользователя или сообщества, фотографии которого нужно получить. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param extended: 1 — возвращать расширенную информацию о фотографиях.
        :param offset: смещение, необходимое для выборки определенного подмножества фотографий. По умолчанию — 0.
        :param count: число фотографий, информацию о которых необходимо получить.
        :param photo_sizes: 1 — будут возвращены размеры фотографий в специальном формате.
        :param no_service_albums: 0 — вернуть все фотографии, включая находящиеся в сервисных альбомах, таких как
            "Фотографии на моей стене" (по умолчанию);  1 — вернуть фотографии только из стандартных альбомов
            пользователя или сообщества.
        :param need_hidden: 1 — возвращает информацию от том, скрыта ли фотография из блока над стеной
            пользователя.
        :param skip_hidden: 1 — не возвращать фотографии, скрытые из блока над стеной пользователя (параметр
            учитывается только при owner_id > 0, параметр no_service_albums игнорируется).

        :return: После успешного выполнения возвращает список объектов photo.  Если был задан параметр
            extended=1, возвращаются дополнительные поля:   likes — количество отметок Мне нравится и информация о
            том, поставил ли лайк текущий пользователь;  reposts — число репостов фотографии.   Если был задан
            параметр photo_sizes=1, вместо полей width и height возвращаются размеры копий фотографии в специальном
            формате.  Если был задан параметр need_hidden=1, будет возвращено поле hidden = 1 в случае, если
            фотография скрыта из блока над стеной пользователя.  Если был задан параметр skip_hidden=1, для объекта
            ответа будет возвращено поле more, принимающее значения 1 или 0, обозначающее наличие или отсутствие
            следующих страниц с фотографиями пользователя. Также для объекта фотографии будет возвращено поле
            real_offset, обозначающее настоящую позицию фотографии из всех фотографий пользователя. Максимальное
            полученное значение этого поля необходимо передать в offset запроса для следующей страницы
        """

        params = locals()
        method_name = self._base_method + 'getAll'
        return self._call(method_name, params)

    def getAllComments(self, owner_id: str = None, album_id: str = None, need_likes: str = None, offset: str = None,
                       count: str = None) -> dict:
        """
        Возвращает отсортированный в антихронологическом порядке список всех комментариев к конкретному альбому или ко
        всем альбомам пользователя.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежат фотографии. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома. Если параметр не задан, то считается, что необходимо получить
            комментарии ко всем альбомам пользователя или сообщества.
        :param need_likes: 1 — будет возвращено дополнительное поле likes. По умолчанию поле likes не
            возвращается.
        :param offset: смещение, необходимое для выборки определенного подмножества комментариев. Если параметр
            не задан, то считается, что он равен 0.
        :param count: количество комментариев, которое необходимо получить. Если параметр не задан, то считается
            что он равен 20. Максимальное значение параметра 100.  Обратите внимание, даже при использовании
            параметра offset для получения доступны только первые 10000 комментариев.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов комментариев в поле items с дополнительным полем pid, содержащим идентификатор фотографии, к
            которой был оставлен комментарий.  Если был задан параметр need_likes=1, возвращается дополнительное поле
            likes:   count — число пользователей, которым понравился комментарий;  user_likes — наличие отметки «Мне
            нравится» от текущего пользователя  (1 — есть, 0 — нет);  can_like — информация о том, может ли текущий
            пользователь поставить отметку «Мне нравится»  (1 — может, 0 — не может)
        """

        params = locals()
        method_name = self._base_method + 'getAllComments'
        return self._call(method_name, params)

    def getById(self, photos: str = None, extended: str = None, photo_sizes: str = None) -> dict:
        """
        Возвращает информацию о фотографиях по их идентификаторам.

        :param photos: перечисленные через запятую идентификаторы, которые представляют собой идущие через знак
            подчеркивания id пользователей, разместивших фотографии, и id самих фотографий. Чтобы получить информацию
            о фотографии в альбоме группы, вместо id пользователя следует указать -id группы. Пример значения photos:
            1_129207899,6492_135055734,  -20629724_271945303  Некоторые фотографии, идентификаторы которых могут быть
            получены через API, закрыты приватностью, и не будут получены. В этом случае следует использовать ключ
            доступа фотографии (access_key) в её идентификаторе. Пример значения photos:
            1_129207899_220df2876123d3542f, 6492_135055734_e0a9bcc31144f67fbd  Поле access_key будет возвращено
            вместе с остальными данными фотографии в методах, которые возвращают фотографии, закрытые приватностью но
            доступные в данном контексте. Например данное поле имеют фотографии, возвращаемые методом newsfeed.get.
        :param extended: 1 — будут возвращены дополнительные поля likes, comments, tags, can_comment, can_repost.
            Поля comments и tags содержат только количество объектов. По умолчанию данные поля не возвращается.
        :param photo_sizes: возвращать ли доступные размеры фотографии в специальном формате.

        :return: После успешного выполнения возвращает массив объектов photo.  Если к фотографии прикреплено
            местоположение, также возвращаются поля lat и long, содержащие географические координаты отметки.  Если
            был задан параметр extended=1, возвращаются дополнительные поля:   likes — количество отметок Мне
            нравится и информация о том, поставил ли лайк текущий пользователь;  comments — количество комментариев к
            фотографии;  tags — количество отметок на фотографии;  can_comment — может ли текущий пользователь
            комментировать фото (1 — может, 0 — не может);  can_repost — может ли текущий пользователь сделать репост
            фотографии (1 — может, 0 — не может).   Если был задан параметр photo_sizes, вместо полей width и height
            возвращаются размеры копий фотографии в специальном формате
        """

        params = locals()
        method_name = self._base_method + 'getById'
        return self._call(method_name, params)

    def getChatUploadServer(self, chat_id: str = None, crop_x: str = None, crop_y: str = None,
                            crop_width: str = None) -> dict:
        """
        Позволяет получить адрес для загрузки обложки чата.

        :param chat_id: идентификатор беседы, для которой нужно загрузить фотографию.
        :param crop_x: координата x для обрезки фотографии (верхний правый угол).
        :param crop_y: координата y для обрезки фотографии (верхний правый угол).
        :param crop_width: ширина фотографии после обрезки в px.

        :return: После успешного выполнения возвращает объект с единственным полем upload_url
        """

        params = locals()
        method_name = self._base_method + 'getChatUploadServer'
        return self._call(method_name, params)

    def getComments(self, owner_id: str = None, photo_id: str = None, need_likes: str = None,
                    start_comment_id: str = None, offset: str = None, count: str = None, sort: str = None,
                    access_key: str = None, extended: str = None, fields: str = None) -> dict:
        """
        Возвращает список комментариев к фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param need_likes: 1 — будет возвращено дополнительное поле likes. По умолчанию: 0.
        :param start_comment_id: идентификатор комментария, начиная с которого нужно вернуть список (подробности
            см. ниже).
        :param offset: смещение, необходимое для выборки определенного подмножества комментариев. По умолчанию:
            0.
        :param count: количество комментариев, которое необходимо получить.
        :param sort: порядок сортировки комментариев. Возможные значения:   asc — от старых к новым;  desc — от
            новых к старым.
        :param access_key: ключ доступа к фотографии.
        :param extended: 1 — в ответе будут возвращены дополнительные поля profiles и groups, содержащие
            информацию о пользователях и сообществах. По умолчанию: 0.
        :param fields: список дополнительных полей профилей, которые необходимо вернуть. См. подробное описание.
            Доступные значения: photo_id, verified, sex, bdate, city, country, home_town, has_photo, photo_50,
            photo_100, photo_200_orig, photo_200, photo_400_orig, photo_max, photo_max_orig, online, lists, domain,
            has_mobile, contacts, site, education, universities, schools, status, last_seen, followers_count,
            common_count, occupation, nickname, relatives, relation, personal, connections, exports, wall_comments,
            activities, interests, music, movies, tv, books, games, about, quotes, can_post, can_see_all_posts,
            can_see_audio, can_write_private_message, can_send_friend_request, is_favorite, is_hidden_from_feed,
            timezone, screen_name, maiden_name, crop_photo, is_friend, friend_status, career, military, blacklisted,
            blacklisted_by_me.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов комментариев в поле items.  Если был задан параметр extended=1, возвращает число результатов в
            поле count, отдельно массив объектов комментариев в поле items, пользователей в поле profiles и сообществ
            в поле groups.  Если был задан параметр need_likes=1, у объектов комментариев возвращается дополнительное
            поле likes:   count — число пользователей, которым понравился комментарий;  user_likes — наличие отметки
            «Мне нравится» от текущего пользователя  (1 — есть, 0 — нет);  can_like — информация о том, может ли
            текущий пользователь поставить отметку «Мне нравится»  (1 — может, 0 — не может).   Если был передан
            параметр start_comment_id, будет также возвращено поле real_offset – итоговое смещение данного
            подмножества комментариев (оно может быть отрицательным, если был указан отрицательный offset)
        """

        params = locals()
        method_name = self._base_method + 'getComments'
        return self._call(method_name, params)

    def getMarketAlbumUploadServer(self, group_id: str = None) -> dict:
        """
        Возвращает адрес сервера для загрузки фотографии подборки товаров в сообществе.

        :param group_id: идентификатор сообщества, для которого необходимо загрузить фотографию подборки товаров.

        :return: После успешного выполнения возвращает объект с единственным полем upload_url
        """

        params = locals()
        method_name = self._base_method + 'getMarketAlbumUploadServer'
        return self._call(method_name, params)

    def getMarketUploadServer(self, group_id: str = None, main_photo: str = None, crop_x: str = None,
                              crop_y: str = None, crop_width: str = None) -> dict:
        """
        Возвращает адрес сервера для загрузки фотографии товара.

        :param group_id: идентификатор сообщества, для которого необходимо загрузить фотографию товара.
        :param main_photo: является ли фотография обложкой товара (1 — фотография для обложки, 0 — дополнительная
            фотография)
        :param crop_x: координата x для обрезки фотографии (верхний правый угол).
        :param crop_y: координата y для обрезки фотографии (верхний правый угол).
        :param crop_width: ширина фотографии после обрезки в px.

        :return: После успешного выполнения возвращает объект с единственным полем upload_url
        """

        params = locals()
        method_name = self._base_method + 'getMarketUploadServer'
        return self._call(method_name, params)

    def getMessagesUploadServer(self, peer_id: str = None) -> dict:
        """
        Возвращает адрес сервера для загрузки фотографии в личное сообщение.

        :param peer_id: идентификатор назначения (для загрузки фотографии в сообщениях сообществ).

        :return: После успешного выполнения возвращает объект с полями upload_url, album_id (id альбома),
            group_id (идентификатор сообщества, если используются сообщения сообщества)
        """

        params = locals()
        method_name = self._base_method + 'getMessagesUploadServer'
        return self._call(method_name, params)

    def getNewTags(self, offset: str = None, count: str = None) -> dict:
        """
        Возвращает список фотографий, на которых есть непросмотренные отметки.

        :param offset: смещение, необходимое для получения определённого подмножества фотографий.
        :param count: количество фотографий, которые необходимо вернуть.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов фотографий в поле items.  Объекты фотографий содержат дополнительные поля:   placer_id —
            идентификатор пользователя, сделавшего отметку;  tag_created — дата создания отметки в формате unixtime;
            tag_id — идентификатор отметки
        """

        params = locals()
        method_name = self._base_method + 'getNewTags'
        return self._call(method_name, params)

    def getOwnerCoverPhotoUploadServer(self, group_id: str = None, crop_x: str = None, crop_y: str = None,
                                       crop_x2: str = None, crop_y2: str = None) -> dict:
        """
        Получает адрес для загрузки обложки сообщества.

        :param group_id: идентификатор сообщества.
        :param crop_x: координата X верхнего левого угла для обрезки изображения.
        :param crop_y: координата Y верхнего левого угла для обрезки изображения.
        :param crop_x2: координата X нижнего правого угла для обрезки изображения.
        :param crop_y2: координата Y нижнего правого угла для обрезки изображения.

        :return: Возвращает объект с единственным полем upload_url, содержащим адрес сервера для загрузки фото
        """

        params = locals()
        method_name = self._base_method + 'getOwnerCoverPhotoUploadServer'
        return self._call(method_name, params)

    def getOwnerPhotoUploadServer(self, owner_id: str = None) -> dict:
        """
        Возвращает адрес сервера для загрузки главной фотографии на страницу пользователя или сообщества.

        :param owner_id: идентификатор сообщества или текущего пользователя. Обратите внимание, идентификатор
            сообщества в параметре owner_id необходимо указывать со знаком "-" — например, owner_id=-1 соответствует
            идентификатору сообщества ВКонтакте API (club1)

        :return: После успешного выполнения возвращает объект с единственным полем upload_url
        """

        params = locals()
        method_name = self._base_method + 'getOwnerPhotoUploadServer'
        return self._call(method_name, params)

    def getTags(self, owner_id: str = None, photo_id: str = None, access_key: str = None) -> dict:
        """
        Возвращает список отметок на фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param access_key: строковой ключ доступа, который может быть получен при получении объекта фотографии.

        :return: После успешного выполнения возвращает массив объектов tag, каждый из которых содержит следующие
            поля:   user_id — идентификатор пользователя, которому соответствует отметка;  id — идентификатор
            отметки;  placer_id — идентификатор пользователя, сделавшего отметку;  tagged_name — название отметки;
            date — дата добавления отметки в формате unixtime;  x, y, x2, y2 — координаты прямоугольной области, на
            которой сделана отметка (верхний левый угол и нижний правый угол) в процентах;  viewed — статус отметки
            (1 — подтвержденная, 0 — неподтвержденная)
        """

        params = locals()
        method_name = self._base_method + 'getTags'
        return self._call(method_name, params)

    def getUploadServer(self, album_id: str = None, group_id: str = None) -> dict:
        """
        Возвращает адрес сервера для загрузки фотографий.

        :param album_id: идентификатор альбома.
        :param group_id: идентификатор сообщества, которому принадлежит альбом (если необходимо загрузить
            фотографию в альбом сообщества).  Если group_id не указан, возвращается адрес для загрузки на стену
            текущего пользователя.

        :return: После успешного выполнения возвращает объект, содержащий следующие поля:   upload_url — адрес
            для загрузки фотографий;  album_id — идентификатор альбома, в который будет загружена фотография;
            user_id — идентификатор пользователя, от чьего имени будет загружено фото
        """

        params = locals()
        method_name = self._base_method + 'getUploadServer'
        return self._call(method_name, params)

    def getUserPhotos(self, user_id: str = None, offset: str = None, count: str = None, extended: str = None,
                      sort: str = None) -> dict:
        """
        Возвращает список фотографий, на которых отмечен пользователь

        :param user_id: идентификатор пользователя, список фотографий для которого нужно получить.
        :param offset: смещение, необходимое для выборки определенного подмножества фотографий. По умолчанию — 0.
        :param count: количество фотографий, которое необходимо получить.
        :param extended: 1 — будут возвращены дополнительные поля likes, comments, tags, can_comment. Поля
            comments и tags содержат только количество объектов. По умолчанию данные поля не возвращается.
        :param sort: сортировка результатов (0 — по дате добавления отметки в порядке убывания, 1 — по дате
            добавления отметки в порядке возрастания).

        :return: После успешного выполнения возвращает список объектов photo.  Если был задан параметр
            extended=1, возвращаются дополнительные поля:   likes — количество отметок Мне нравится и информация о
            том, поставил ли лайк текущий пользователь;  comments — количество комментариев к фотографии;  tags —
            количество отметок на фотографии;  can_comment — может ли текущий пользователь комментировать фото (1 —
            может, 0 — не может).   Если был задан параметр photo_sizes=1, вместо полей width и height возвращаются
            размеры копий фотографии в специальном формате
        """

        params = locals()
        method_name = self._base_method + 'getUserPhotos'
        return self._call(method_name, params)

    def getWallUploadServer(self, group_id: str = None) -> dict:
        """
        Возвращает адрес сервера для загрузки фотографии на стену пользователя или сообщества.

        :param group_id: идентификатор сообщества, на стену которого нужно загрузить фото (без знака «минус»).

        :return: После успешного выполнения возвращает объект с полями upload_url, album_id, user_id
        """

        params = locals()
        method_name = self._base_method + 'getWallUploadServer'
        return self._call(method_name, params)

    def makeCover(self, owner_id: str = None, photo_id: str = None, album_id: str = None) -> dict:
        """
        Делает фотографию обложкой альбома.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии. Фотография должна находиться в альбоме album_id.
        :param album_id: идентификатор альбома.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'makeCover'
        return self._call(method_name, params)

    def move(self, owner_id: str = None, target_album_id: str = None, photo_id: str = None) -> dict:
        """
        Переносит фотографию из одного альбома в другой.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param target_album_id: идентификатор альбома, в который нужно переместить фотографию.
        :param photo_id: идентификатор фотографии, которую нужно перенести.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'move'
        return self._call(method_name, params)

    def putTag(self, owner_id: str = None, photo_id: str = None, user_id: str = None, x: str = None, y: str = None,
               x2: str = None, y2: str = None) -> dict:
        """
        Добавляет отметку на фотографию.

        :param owner_id: идентификатор пользователя, которому принадлежит фотография.
        :param photo_id: идентификатор фотографии.
        :param user_id: идентификатор пользователя, которого нужно отметить на фотографии.
        :param x: координата верхнего левого угла области с отметкой в % от ширины фотографии.
        :param y: координата верхнего левого угла области с отметкой в % от высоты фотографии.
        :param x2: координата правого нижнего угла области с отметкой в % от ширины фотографии.
        :param y2: координата правого нижнего угла области с отметкой в % от высоты фотографии.

        :return: После успешного выполнения возвращает идентификатор созданной отметки (tag id)
        """

        params = locals()
        method_name = self._base_method + 'putTag'
        return self._call(method_name, params)

    def removeTag(self, owner_id: str = None, photo_id: str = None, tag_id: str = None) -> dict:
        """
        Удаляет отметку с фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param tag_id: идентификатор отметки.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeTag'
        return self._call(method_name, params)

    def reorderAlbums(self, owner_id: str = None, album_id: str = None, before: str = None, after: str = None) -> dict:
        """
        Меняет порядок альбома в списке альбомов пользователя.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит альбом. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома.
        :param before: идентификатор альбома, перед которым следует поместить альбом.
        :param after: идентификатор альбома, после которого следует поместить альбом.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reorderAlbums'
        return self._call(method_name, params)

    def reorderPhotos(self, owner_id: str = None, photo_id: str = None, before: str = None, after: str = None) -> dict:
        """
        Меняет порядок фотографии в списке фотографий альбома пользователя.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.
        :param before: идентификатор фотографии, перед которой следует поместить фотографию. Если параметр не
            указан, фотография будет помещена последней.
        :param after: идентификатор фотографии, после которой следует поместить фотографию. Если параметр не
            указан, фотография будет помещена первой.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reorderPhotos'
        return self._call(method_name, params)

    def report(self, owner_id: str = None, photo_id: str = None, reason: str = None) -> dict:
        """
        Позволяет пожаловаться на фотографию.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография.
        :param photo_id: идентификатор фотографии.
        :param reason: причина жалобы:   0 — спам;  1 — детская порнография;  2 — экстремизм;  3 — насилие;  4 —
            пропаганда наркотиков;  5 — материал для взрослых;  6 — оскорбление.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'report'
        return self._call(method_name, params)

    def reportComment(self, owner_id: str = None, comment_id: str = None, reason: str = None) -> dict:
        """
        Позволяет пожаловаться на комментарий к фотографии.

        :param owner_id: идентификатор владельца фотографии к которой оставлен комментарий.
        :param comment_id: идентификатор комментария.
        :param reason: причина жалобы:   0 — спам;  1 — детская порнография;  2 — экстремизм;  3 — насилие;  4 —
            пропаганда наркотиков;  5 — материал для взрослых;  6 — оскорбление.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reportComment'
        return self._call(method_name, params)

    def restore(self, owner_id: str = None, photo_id: str = None) -> dict:
        """
        Восстанавливает удаленную фотографию.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param photo_id: идентификатор фотографии.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'restore'
        return self._call(method_name, params)

    def restoreComment(self, owner_id: str = None, comment_id: str = None) -> dict:
        """
        Восстанавливает удаленный комментарий к фотографии.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит фотография. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param comment_id: идентификатор удаленного комментария.

        :return: После успешного выполнения возвращает 1 (0, если комментарий с таким идентификатором не является
            удаленным)
        """

        params = locals()
        method_name = self._base_method + 'restoreComment'
        return self._call(method_name, params)

    def save(self, album_id: str = None, group_id: str = None, server: str = None, photos_list: str = None,
             hash: str = None, latitude: str = None, longitude: str = None, caption: str = None) -> dict:
        """
        Сохраняет фотографии после успешной загрузки.

        :param album_id: идентификатор альбома, в который необходимо сохранить фотографии.
        :param group_id: идентификатор сообщества, в которое необходимо сохранить фотографии.
        :param server: параметр, возвращаемый в результате загрузки фотографий на сервер.
        :param photos_list: параметр, возвращаемый в результате загрузки фотографий на сервер.
        :param hash: параметр, возвращаемый в результате загрузки фотографий на сервер.
        :param latitude: географическая широта, заданная в градусах (от -90 до 90);
        :param longitude: географическая долгота, заданная в градусах (от -180 до 180);
        :param caption: текст описания фотографии (максимум 2048 символов).

        :return: После успешного выполнения возвращает массив объектов фотографий
        """

        params = locals()
        method_name = self._base_method + 'save'
        return self._call(method_name, params)

    def saveMarketAlbumPhoto(self, group_id: str = None, photo: str = None, server: str = None,
                             hash: str = None) -> dict:
        """
        Сохраняет фотографии после успешной загрузки на URI, полученный методом photos.getMarketAlbumUploadServer.

        :param group_id: идентификатор группы, для которой нужно загрузить фотографию.
        :param photo: параметр, возвращаемый в результате загрузки фотографии на сервер.  Минимальный размер
            фотографии — 1280x720 пикселей.
        :param server: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param hash: параметр, возвращаемый в результате загрузки фотографии на сервер.

        :return: После успешного выполнения возвращает массив, содержащий объект с загруженной фотографией
        """

        params = locals()
        method_name = self._base_method + 'saveMarketAlbumPhoto'
        return self._call(method_name, params)

    def saveMarketPhoto(self, group_id: str = None, photo: str = None, server: str = None, hash: str = None,
                        crop_data: str = None, crop_hash: str = None) -> dict:
        """
        Сохраняет фотографии после успешной загрузки на URI, полученный методом photos.getMarketUploadServer.

        :param group_id: идентификатор группы, для которой нужно загрузить фотографию.
        :param photo: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param server: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param hash: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param crop_data: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param crop_hash: параметр, возвращаемый в результате загрузки фотографии на сервер.

        :return: После успешного выполнения возвращает массив, содержащий объект с загруженной фотографией
        """

        params = locals()
        method_name = self._base_method + 'saveMarketPhoto'
        return self._call(method_name, params)

    def saveMessagesPhoto(self, photo: str = None, server: str = None, hash: str = None) -> dict:
        """
        Сохраняет фотографию после успешной загрузки на URI, полученный методом photos.getMessagesUploadServer.

        :param photo: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param server: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param hash: параметр, возвращаемый в результате загрузки фотографии на сервер.

        :return: После успешного выполнения возвращает массив с загруженной фотографией, возвращённый объект
            имеет поля id, pid, aid, owner_id, src, src_big, src_small, created. В случае наличия фотографий в
            высоком разрешении также будут возвращены адреса с названиями src_xbig и src_xxbig
        """

        params = locals()
        method_name = self._base_method + 'saveMessagesPhoto'
        return self._call(method_name, params)

    def saveOwnerCoverPhoto(self, hash: str = None, photo: str = None) -> dict:
        """
        Сохраняет изображение для обложки сообщества после успешной загрузки.

        :param hash: параметр hash, полученный в результате загрузки фотографии на сервер.
        :param photo: параметр photo, полученный в результате загрузки фотографии на сервер.

        :return: Возвращает массив images объектов, описывающих копии загруженной фотографии. Каждый объект
            содержит следующие поля:   url (string) — URL копии фотографии;  width (integer) — ширина копии в px;
            height (integer) — высота копии в px
        """

        params = locals()
        method_name = self._base_method + 'saveOwnerCoverPhoto'
        return self._call(method_name, params)

    def saveOwnerPhoto(self, server: str = None, hash: str = None, photo: str = None) -> dict:
        """
        Позволяет сохранить главную фотографию пользователя или сообщества.

        :param server: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param hash: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param photo: параметр, возвращаемый в результате загрузки фотографии на сервер.

        :return: После успешного выполнения возвращает объект, содержащий поля photo_hash и photo_src (при работе
            через VK.api метод вернёт поля photo_src, photo_src_big, photo_src_small). Параметр photo_hash необходим
            для подтверждения пользователем изменения его фотографии через вызов метода saveProfilePhoto Javascript
            API. Поле photo_src содержит путь к загруженной фотографии
        """

        params = locals()
        method_name = self._base_method + 'saveOwnerPhoto'
        return self._call(method_name, params)

    def saveWallPhoto(self, user_id: str = None, group_id: str = None, photo: str = None, server: str = None,
                      hash: str = None, latitude: str = None, longitude: str = None, caption: str = None) -> dict:
        """
        Сохраняет фотографии после успешной загрузки на URI, полученный методом photos.getWallUploadServer.

        :param user_id: идентификатор пользователя, на стену которого нужно сохранить фотографию.
        :param group_id: идентификатор сообщества, на стену которого нужно сохранить фотографию.
        :param photo: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param server: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param hash: параметр, возвращаемый в результате загрузки фотографии на сервер.
        :param latitude: географическая широта, заданная в градусах (от -90 до 90);
        :param longitude: географическая долгота, заданная в градусах (от -180 до 180);
        :param caption: текст описания фотографии (максимум 2048 символов).

        :return: После успешного выполнения возвращает массив, содержащий объект с загруженной фотографией
        """

        params = locals()
        method_name = self._base_method + 'saveWallPhoto'
        return self._call(method_name, params)

    def search(self, q: str = None, lat: str = None, long: str = None, start_time: str = None, end_time: str = None,
               sort: str = None, offset: str = None, count: str = None, radius: str = None) -> dict:
        """
        Осуществляет поиск изображений по местоположению или описанию.

        :param q:  поискового запроса, например, Nature.
        :param lat: географическая широта отметки, заданная в градусах (от -90 до 90).
        :param long: географическая долгота отметки, заданная в градусах (от -180 до 180).
        :param start_time: время в формате unixtime, не раньше которого должны были быть загружены найденные
            фотографии.
        :param end_time: время в формате unixtime, не позже которого должны были быть загружены найденные
            фотографии.
        :param sort: сортировка результатов. Возможные значения:   1 — по количеству отметок «Мне нравится»;  0 —
            по дате добавления фотографии.
        :param offset: смещение относительно первой найденной фотографии для выборки определенного подмножества.
        :param count: количество возвращаемых фотографий.  Обратите внимание — даже при использовании параметра
            offset для получения информации доступны не больше 3000 результатов.
        :param radius: радиус поиска в метрах. (работает очень приближенно, поэтому реальное расстояние до цели
            может отличаться от заданного). Может принимать значения: 10, 100, 800, 6000, 50000

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов фотографий в поле items
        """

        params = locals()
        method_name = self._base_method + 'search'
        return self._call(method_name, params)



from ApiMethod import ApiMethod
from vk_objects import Media
from typing import List, Dict, Union, Optional
import time


class Messages(ApiMethod):
    def __init__(self, access_token, v, session, calls_per_second):
        super(Messages, self).__init__(access_token, v, session, calls_per_second)
        self._base_method = 'messages.'

    def addChatUser(self, chat_id: int, user_id: int, visible_messages_count: int = None) -> int:
        """
        Добавляет в мультидиалог нового пользователя. тобы пользователю вернуться в беседу, которую он сам покинул,
        достаточно отправить сообщение в неё (если есть свободные места), либо вызвать этот метод, передав свой
        идентификатор в параметре user_id.

        :param chat_id: идентификатор беседы.
        :param user_id: идентификатор пользователя, которого необходимо включить в беседу.
        :param visible_messages_count:

        :return: После успешного выполнения возвращает 1.
        """

        params = locals()
        method_name = self._base_method + 'addChatUser'
        return self._call(method_name, params)

    def allowMessagesFromGroup(self, group_id: int, key: str = None) -> int:
        """
        Позволяет разрешить отправку сообщений от сообщества текущему пользователю.

        :param group_id: идентификатор сообщества.
        :param key: произвольная строка. Этот параметр можно использовать для идентификации пользователя.
            Его значение будет возвращено в событии message_allow Callback API.

        :return: После успешного выполнения возвращает 1.
        """

        params = locals()
        method_name = self._base_method + 'allowMessagesFromGroup'
        return self._call(method_name, params)

    def createChat(self, user_ids: List[int], title: str = None, group_id: int = None) -> dict:
        """
        Создаёт беседу с несколькими участниками.

        :param user_ids: идентификаторы пользователей, которых нужно включить в мультидиалог. Должны быть в
            друзьях у текущего пользователя.
        :param title: название беседы.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает идентификатор созданного чата (chat_id).
        """

        params = locals()
        params['user_ids'] = str([user_ids]).strip('[]')
        method_name = self._base_method + 'createChat'
        return self._call(method_name, params)

    def delete(self, message_ids: List[int], spam: bool = None, group_id: int = None,
               delete_for_all: bool = None) -> int:
        """
        Удаляет сообщение.

        :param message_ids: список идентификаторов сообщений, разделённых через запятую.
        :param spam: пометить сообщения как спам.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).
        :param delete_for_all: 1 — если сообщение нужно удалить для получателей (если с момента отправки
            сообщения прошло не более 24 часов ).

        :return: После успешного выполнения возвращает 1 для каждого удаленного сообщения.
        """

        params = locals()
        params['message_ids'] = str([message_ids]).strip('[]')
        method_name = self._base_method + 'delete'
        return self._call(method_name, params)

    # TODO: объект мультидиалога
    def deleteChatPhoto(self, chat_id: int, group_id: int = None) -> int:
        """
        Позволяет удалить фотографию мультидиалога.

        :param chat_id: идентификатор беседы.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает объект, содержащий следующие поля:
            message_id — идентификатор отправленного системного сообщения;
            chat — объект мультидиалога.
        """

        params = locals()
        method_name = self._base_method + 'deleteChatPhoto'
        return self._call(method_name, params)

    def deleteConversation(self, user_id: int = None, peer_id: int = None, group_id: int = None) -> dict:
        """
        Удаляет беседу.

        :param user_id: идентификатор пользователя. Если требуется очистить историю беседы, используйте
            peer_id.
        :param peer_id: идентификатор назначения. Для групповой беседы: 2000000000 + id беседы.
            Для сообщества: -id сообщества.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает поле last_deleted_id, содержащее идентификатор
            последнего удалённого сообщения в переписке.
        """

        params = locals()
        method_name = self._base_method + 'deleteConversation'
        return self._call(method_name, params)

    def denyMessagesFromGroup(self, group_id: int) -> int:
        """
        Позволяет запретить отправку сообщений от сообщества текущему пользователю.

        :param group_id: идентификатор сообщества.

        :return: После успешного выполнения возвращает 1.
        """

        params = locals()
        method_name = self._base_method + 'denyMessagesFromGroup'
        return self._call(method_name, params)

    # TODO make attachments
    def edit(self, peer_id: int, message: str = None, message_id: int = None, lat: float = None, long: float = None,
             attachment: int = None, keep_forward_messages: bool = None, keep_snippets: bool = None,
             group_id: int = None, dont_parse_links: bool = None) -> int:
        """
        Редактирует сообщение.

        :param peer_id: идентификатор назначения. Для пользователя: id пользователя.
            Для групповой беседы: 2000000000 + id беседы.
            Для сообщества: -id сообщества.
        :param message: текст сообщения. Обязательный параметр, если не задан параметр attachment.
        :param message_id: идентификатор сообщения.
        :param lat: географическая широта (от -90 до 90).
        :param long: географическая долгота (от -180 до 180).
        :param attachment: медиавложения к личному сообщению, перечисленные через запятую.
            Каждое прикрепление представлено в формате: <type><owner_id>_<media_id>
                <type> — тип медиавложения:
                    photo — фотография;
                    video — видеозапись;
                    audio — аудиозапись;
                    doc — документ;
                    wall — запись на стене;
                    market — товар.
                <owner_id> — идентификатор владельца медиавложения (обратите внимание, если объект находится в
                сообществе, этот параметр должен быть отрицательным).
                <media_id> — идентификатор медиавложения.
            Например: photo100172_166443618
            Параметр является обязательным, если не задан параметр message.
            В случае, если прикрепляется объект, принадлежащий другому пользователю следует добавлять к вложению его
            access_key в формате <type><owner_id>_<media_id>_<access_key>,
            Например: video85635407_165186811_69dff3de4372ae9b6e
        :param keep_forward_messages: 1, чтобы сохранить прикреплённые пересланные сообщения.
        :param keep_snippets: 1, чтобы сохранить прикреплённые внешние ссылки (сниппеты).
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).
        :param dont_parse_links: 1 — не создавать сниппет ссылки из сообщения

        :return: После успешного выполнения возвращает 1.
        """

        params = locals()
        method_name = self._base_method + 'edit'
        return self._call(method_name, params)

    def editChat(self, chat_id: int, title: str) -> int:
        """
        Изменяет название беседы.

        :param chat_id: идентификатор беседы.
        :param title: новое название для беседы.

        :return: После успешного выполнения возвращает 1.
        """

        params = locals()
        method_name = self._base_method + 'editChat'
        return self._call(method_name, params)

    # TODO сделать объект сообщения
    def getByConversationMessageId(self, peer_id: int, conversation_message_ids: int, extended: bool = None,
                                   fields: str = None, group_id: int = None) -> dict:
        """
        Возвращает сообщения по их идентификаторам в рамках беседы или диалога.

        :param peer_id: идентификаторы назначений, разделённые запятой. Для пользователя: id пользователя.
            Для групповой беседы: 2000000000 + id беседы (недоступно с ключом сообщества)
            Для сообщества: -id сообщества.
        :param conversation_message_ids: идентификаторы сообщений. Максимум 100 идентификаторов.
        :param extended: 1 — возвращать дополнительные поля.
        :param fields: дополнительные поля пользователей и сообществ, которые необходимо вернуть в ответе.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и
            массив объектов, описывающих сообщения, в поле items.
        """

        params = locals()
        method_name = self._base_method + 'getByConversationMessageId'
        return self._call(method_name, params)

    # TODO сделать объект сообщения
    def getById(self, message_ids: List[int], preview_length: bool = None, extended: bool = None, fields: str = None,
                group_id: int = None) -> dict:
        """
        Возвращает сообщения по их идентификаторам.

        :param message_ids: идентификаторы сообщений. Максимум 100 идентификаторов.
        :param preview_length: количество символов, по которому нужно обрезать сообщение. Укажите 0, если Вы
            не хотите обрезать сообщение. (по умолчанию сообщения не обрезаются).
        :param extended: 1 — возвращать дополнительные поля.
        :param fields: список дополнительных полей для пользователей и сообществ.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и
            массив объектов, описывающих сообщения, в поле items.
        """

        params = locals()
        params['message_ids'] = str([message_ids]).strip('[]')
        method_name = self._base_method + 'getById'
        return self._call(method_name, params)

    # TODO: сделать объект мультидиалога
    def getChat(self, chat_id: int = None, chat_ids: List[int] = None, fields: str = None,
                name_case: str = None) -> Union[List[dict], dict]:
        """
        Возвращает информацию о беседе.

        :param chat_id: идентификатор беседы.
        :param chat_ids: список идентификаторов бесед.
        :param fields: список дополнительных полей профилей, которые необходимо вернуть. Доступные
            значения: nickname, screen_name, sex, bdate, city, country, timezone, photo_50, photo_100,
            photo_200_orig, has_mobile, contacts, education, online, counters, relation, last_seen, status,
            can_write_private_message, can_see_all_posts, can_post, universities
        :param name_case: падеж для склонения имени и фамилии пользователя. Возможные значения:
            nom — именительный;
            gen — родительный;
            dat — дательный;
            acc — винительный;
            ins — творительный;
            abl — предложный.
            По умолчанию: nom.

        :return: После успешного выполнения возвращает объект (или список объектов) мультидиалога. Если был
            задан параметр fields, поле users содержит список объектов пользователей с дополнительным полем
            invited_by, содержащим идентификатор пользователя, пригласившего в беседу.
        """

        params = locals()
        if chat_ids:
            params['chat_ids'] = str([chat_ids]).strip('[]')
        method_name = self._base_method + 'getChat'
        return self._call(method_name, params)

    def getChatPreview(self, peer_id: int, link: str = None, fields: str = None) -> dict:
        """
        Получает данные для превью чата с приглашением по ссылке.

        :param peer_id:
        :param link: ссылка-приглашение.
        :param fields: список полей профилей, данные о которых нужно получить. Полный список смотрите на
            этой странице. (https://vk.com/dev/objects/user)

        :return: Возвращает объект, который содержит следующие поля:
            preview - информация о чате. Объект, который содержит поля:
                admin_id (object) — идентификатор создателя чата;
                members (array) — массив идентификаторов участников чата;
                title (string) — название чата;
                photo (object) — обложка чата. Объект, который содержит поля:
                    photo_50 (string) — URL копии фотографии с шириной 50 px;
                    photo_100 (string) — URL копии фотографии с шириной 100 px;
                    photo_200 (string) — URL копии фотографии с шириной 200px.
                local_id (integer) — идентификатор чата для текущего пользователя.
            profiles (array) - массив объектов пользователей;
            groups (array) - массив объектов сообществ;
            emails (array) - массив объектов, описывающих e-mail. Каждый из объектов содержит поля:
                id (integer) — идентификатор e-mail;
                address (string) — адрес e-mail.
        """

        params = locals()
        method_name = self._base_method + 'getChatPreview'
        return self._call(method_name, params)

    def getConversationMembers(self, peer_id: int, fields: str = None, group_id: int = None) -> dict:
        """
        Позволяет получить список участников беседы.

        :param peer_id: идентификатор назначения. Для пользователя: id пользователя.
            Для групповой беседы: 2000000000 + id беседы. Для сообщества: -id сообщества.
        :param fields: дополнительные поля пользователей и сообществ, которые необходимо вернуть в ответе.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: Возвращает объект, который содержит следующие поля:
            count (integer) - число участников беседы.
            items (array) - участники беседы. Массив объектов, каждый из которых содержит поля:
                member_id (integer) — идентификатор участника беседы;
                invited_by (integer) — идентификатор пользователя, который пригласил участника;
                join_date (integer) — дата добавления в беседу;
                is_admin (boolean) — является ли пользователь администратором;
                can_kick (boolean) — может ли текущий пользователь исключить участника.
            profiles (array) - массив объектов пользователей.
            groups (array) - массив объектов сообществ.
        """

        params = locals()
        method_name = self._base_method + 'getConversationMembers'
        return self._call(method_name, params)

    # TODO: сделать объект сообществ
    def getConversations(self, offset, count, filter, extended, start_message_id, fields, group_id) -> dict:
        """
        Возвращает список бесед пользователя.

        :param offset: смещение, необходимое для выборки определенного подмножества результатов.
        :param count: максимальное число результатов, которые нужно получить.
        :param filter: фильтр. Возможные значения:
            all — все беседы;
            unread — беседы с непрочитанными сообщениями;
            important — беседы, помеченные как важные (только для сообщений сообществ);
            unanswered — беседы, помеченные как неотвеченные (только для сообщений сообществ).
            По умолчанию: all.
        :param extended: 1 — возвращать дополнительные поля для пользователей и сообществ.
        :param start_message_id: идентификатор сообщения, начиная с которого нужно возвращать беседы.
        :param fields: список дополнительных полей для пользователей и сообществ.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: Возвращает объект, который содержит следующие поля:
         count (integer) - число результатов.
         items (array) - беседы. Массив объектов, каждый из которых содержит поля:
            conversation (object) — объект беседы.
            last_message (object) — объект, описывающий последнее сообщение в беседе.
        unread_count (integer) - число непрочитанных бесед.
        profiles (array) - массив объектов пользователей.
        groups (array) массив объектов сообществ.
        """

        params = locals()
        method_name = self._base_method + 'getConversations'
        return self._call(method_name, params)

    # TODO: сделать объект бемед
    def getConversationsById(self, peer_ids: Union[List[int], int], extended: bool = None, fields: str = None,
                             group_id: int = None) -> dict:
        """
        Позволяет получить беседу по её идентификатору.

        :param peer_ids: идентификаторы назначений, разделённые запятой. Для пользователя: id
            пользователя. Для групповой беседы: 2000000000 + id беседы. Для сообщества: -id сообщества.
            Максимум – 100 идентификаторов.
        :param extended: 1 — возвращать дополнительные поля.
        :param fields: дополнительные поля пользователей и сообществ, которые необходимо вернуть в ответе.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: Возвращает общее число результатов в поле count (integer) и массив объектов бесед в поле items.
        """

        params = locals()
        params['peer_ids'] = str([peer_ids]).strip('[]')
        method_name = self._base_method + 'getConversationsById'
        return self._call(method_name, params)

    # TODO объект сообщений
    def getHistory(self, offset: int = None, count: int = None, user_id: int = None, peer_id: int = None,
                   start_message_id: int = None, rev: bool = None, extended: bool = None,
                   fields: str = None, group_id: int = None) -> dict:
        """
        Возвращает историю сообщений для указанного диалога.

        :param offset: смещение, необходимое для выборки определенного подмножества сообщений, должен быть >= 0,
        если не передан параметр start_message_id, и должен быть <= 0, если передан.
        :param count: количество сообщений, которое необходимо получить (но не более 200).
        :param user_id: идентификатор пользователя, историю переписки с которым необходимо вернуть.
        :param peer_id: идентификатор назначения. Для пользователя: id пользователя. Для групповой
            беседы: 2000000000 + id беседы. (недоступно для вызовов от имени сообщества) Для сообщества: -id
            сообщества.
        :param start_message_id: если значение > 0, то это идентификатор сообщения, начиная с которого нужно
            вернуть историю переписки, если передано значение 0 то вернутся сообщения с самого начала переписки,
            если же передано значение -1, то к значению параметра offset прибавляется количество входящих
            непрочитанных сообщений в конце диалога (подробности см. ниже)
        :param rev: 1 – возвращать сообщения в хронологическом порядке. 0 – возвращать сообщения в обратном
            хронологическом порядке (по умолчанию).
        :param extended: Если указать в качестве этого параметра 1, то будет возвращена информация о
            пользователях, являющихся авторами сообщений. По умолчанию 0.
        :param fields: список дополнительных полей профилей, которые необходимо вернуть.
            См. https://vk.com/dev/objects/user
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает список объектов сообщений. В случае наличия в этом
            диалоге непрочитанных входящих сообщений, дополнительно (после поля count, перед полем items) в
            список будет добавлено поле unread с количеством непрочитанных входящих сообщений. Поля in_read и
            out_read являются идентификаторами прочтения. in_read возвращает идентификатор последнего
            прочитанного сообщения текущим пользователем, а out_read возвращает идентификатор последнего
            прочитанного сообщения собеседником. При переданном start_message_id в случае, если не было
            возвращено последнее сообщение в истории переписки (то есть сколько-то сообщений было пропущено),
            количество пропущенных сообщений (то есть реальное значение offset, которое использовалось для
            получения интервала истории) будет возвращено в поле skipped (вместе с count, items и иногда
            unread).
        """

        params = locals()
        method_name = self._base_method + 'getHistory'
        return self._call(method_name, params)

    def getHistoryAttachments(self, peer_id: str = None, media_type: str = None, start_from: str = None,
                              count: str = None, photo_sizes: str = None, fields: str = None, group_id: str = None,
                              preserve_order: str = None, max_forwards_level: str = None) -> dict:
        """
        Возвращает материалы диалога или беседы.

        :param peer_id: идентификатор назначения. Для групповой беседы:  2000000000 + id беседы.   Для
            пользователя:  id пользователя.   Для сообщества:  -id сообщества.
        :param media_type: тип материалов, который необходимо вернуть.  Доступные значения:   photo — фотографии;
            video — видеозаписи;  audio — аудиозаписи;  doc — документы;  link — ссылки;  market — товары;  wall —
            записи;  share — ссылки, товары и записи.   Обратите внимание — существует ограничение по дате отправки
            вложений. Так, для получения доступны вложения типов photo,video,audio,doc , отправленные не ранее
            25.03.2013, link — не ранее 20.05.13, market,wall — 01.02.2016.
        :param start_from: смещение, необходимое для выборки определенного подмножества объектов.
        :param count: количество объектов, которое необходимо получить (но не более 200).
        :param photo_sizes: параметр, указывающий нужно ли возвращать ли доступные размеры фотографии в
            специальном формате.
        :param fields: дополнительные поля профилей пользователей и сообществ, которые необходимо вернуть в
            ответе.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).
        :param preserve_order: параметр, указывающий нужно ли возвращать вложения в оригинальном порядке.
        :param max_forwards_level: максимальная глубина вложенности пересланных сообщений.

        :return: После успешного выполнения возвращает массив items, содержащий объекты, которые описывают
            вложения, а также дополнительное поле next_from, содержащее новое значение start_from. Каждый объект из
            массива items содержит следующие поля:  message_id  integerидентификатор сообщения, в котором было
            отправлено вложение. attachment  objectинформация о вложении. Объект, который содержит следующие поля:
            type (string) — тип вложения. Возможные значения:   photo — фотография;  video — видеозапись;  audio —
            аудиозапись;  doc — документ;  link — ссылка;  market — товар;  wall — запись.   photo, video, audio,
            doc, link, market, wall (название поля зависит от значения поля type) — описание вложения. Содержит,
            соответственно, объект, описывающий фотографию, видеозапись, аудиозапись, документ, ссылку, товар или
            запись на стене
        """

        params = locals()
        method_name = self._base_method + 'getHistoryAttachments'
        return self._call(method_name, params)

    def getImportantMessages(self, count: str = None, offset: str = None, start_message_id: str = None,
                             preview_length: str = None, fields: str = None, extended: str = None,
                             group_id: str = None) -> dict:
        """
        Возвращает список важных сообщений пользователя.

        :param count: максимальное число результатов, которые нужно получить.
        :param offset: смещение, необходимое для выборки определенного подмножества результатов.
        :param start_message_id: идентификатор сообщения, начиная с которого нужно возвращать список.
        :param preview_length:
        :param fields: список дополнительных полей для пользователей и сообществ.
        :param extended: 1 — возвращать дополнительные поля для пользователей и сообществ.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return
        """

        params = locals()
        method_name = self._base_method + 'getImportantMessages'
        return self._call(method_name, params)

    def getInviteLink(self, peer_id: str = None, reset: str = None, group_id: str = None) -> dict:
        """
        Получает ссылку для приглашения пользователя в беседу.

        :param peer_id: идентификатор назначения.  Для групповой беседы:  2000000000 + id беседы.
        :param reset: 1 — сгенерировать новую ссылку, сбросив предыдущую. 0 — получить предыдущую ссылку.
        :param group_id:

        :return: Возвращает объект с единственным полем link (string), которое содержит ссылку для приглашения в
            беседу
        """

        params = locals()
        method_name = self._base_method + 'getInviteLink'
        return self._call(method_name, params)

    def getLastActivity(self, user_id: str = None) -> dict:
        """
        Возвращает текущий статус и дату последней активности указанного пользователя.

        :param user_id: идентификатор пользователя, информацию о последней активности которого требуется
            получить.

        :return: Возвращает объект, содержащий следующие поля:   online — текущий статус пользователя (1 — в
            сети, 0 — не в сети);  time — дата последней активности пользователя в формате unixtime
        """

        params = locals()
        method_name = self._base_method + 'getLastActivity'
        return self._call(method_name, params)

    def getLongPollHistory(self, ts: str = None, pts: str = None, preview_length: str = None, onlines: str = None,
                           fields: str = None, events_limit: str = None, msgs_limit: str = None, max_msg_id: str = None,
                           group_id: str = None, lp_version: str = None, last_n: str = None,
                           credentials: str = None) -> dict:
        """
        Возвращает обновления в личных сообщениях пользователя.

        :param ts: последнее значение параметра ts, полученное от Long Poll сервера или с помощью метода
            messages.getLongPollServer
        :param pts: последнее значение параметра new_pts, полученное от Long Poll сервера, используется для
            получения действий, которые хранятся всегда.
        :param preview_length: количество символов, по которому нужно обрезать сообщение. Укажите 0, если Вы не
            хотите обрезать сообщение. (по умолчанию сообщения не обрезаются).
        :param onlines: 1 — возвращать в числе прочих события 8 и 9 (пользователь стал онлайн/оффлайн).
            Учитывается только при использовании ts.
        :param fields: список дополнительных полей профилей, которые необходимо вернуть. См. подробное описание.
            Доступные значения: sex, bdate, city, country, photo_50, photo_100, photo_200_orig, photo_200,
            photo_400_orig, photo_max, photo_max_orig, photo_id, online, online_mobile, domain, has_mobile, contacts,
            connections, site, education, universities, schools, can_post, can_see_all_posts, can_see_audio,
            can_write_private_message, status, last_seen, common_count, relation, relatives, counters, screen_name,
            maiden_name, timezone, occupation, activities, interests, music, movies, tv, books, games, about, quotes,
            personal, friend_status, military, career
        :param events_limit: лимит по количеству всех событий в истории. Обратите внимание, параметры
            events_limit и msgs_limit применяются совместно. Число результатов в ответе ограничивается первым
            достигнутым лимитом.
        :param msgs_limit: лимит по количеству событий с сообщениями в истории. Обратите внимание, параметры
            events_limit и msgs_limit применяются совместно. Число результатов в ответе ограничивается первым
            достигнутым лимитом.
        :param max_msg_id: максимальный идентификатор сообщения среди уже имеющихся в локальной копии. Необходимо
            учитывать как сообщения, полученные через методы API (например messages.getDialogs, messages.getHistory),
            так и данные, полученные из Long Poll сервера (события с кодом 4).
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).
        :param lp_version: версия Long Poll.
        :param last_n:
        :param credentials:

        :return: Возвращает объект, который содержит поля history, messages, а также groups — массив объектов
            сообществ и profiles — массив объектов пользователей.  Поле history представляет из себя массив,
            аналогичный полю updates, получаемому от Long Poll сервера, за некоторыми исключениями: для событий с
            кодом 4 (добавление нового сообщения) отсутствуют все поля, кроме первых трёх, а также отсутствуют
            события с кодами 8, 9 (друг появился/пропал из сети) и 61, 62 (набор текста в диалоге/беседе).  Поле
            messages представляет из себя массив личных сообщений – объектов message, которые встречались среди
            событий с кодом 4 (добавление нового сообщения) из поля history. Каждый объект message содержит набор
            полей, описание которых доступно здесь. Первый элемент массива представляет собой общее количество
            сообщений.   Ограничения В случае, если ts слишком старый (больше суток), а max_msg_id не передан, метод
            может вернуть ошибку 10 (Internal server error).  Если количество событий превышает значение events_limit
            или количество событий с сообщениями превышает значение msgs_limit, ответ содержит дополнительное поле
            more со значением 1 — это означает, что нужно запросить оставшиеся данные с помощью запроса с параметром
            max_msg_id. Обратите внимание, что параметры events_limit и msgs_limit применяются совместно — число
            объектов в результате не превышает значения меньшего из этих параметров.  Ошибки с кодами 907 и 908
            означают, что нужно получить новое значение pts (''ts''') и вызвать метод повторно с новыми значениями,
            поскольку данных для переданных значений не существует
        """

        params = locals()
        method_name = self._base_method + 'getLongPollHistory'
        return self._call(method_name, params)

    def getLongPollServer(self, need_pts: str = None, group_id: str = None, lp_version: str = None) -> dict:
        """
        Возвращает данные, необходимые для подключения к Long Poll серверу.

        :param need_pts: 1 — возвращать поле pts, необходимое для работы метода messages.getLongPollHistory
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).
        :param lp_version: версия для подключения к Long Poll. Актуальная версия: 3.  Подробную информацию об
            изменениях в версиях Вы найдёте на этой странице.

        :return: Возвращает объект, который содержит поля key, server, ts.  Используя эти данные, Вы можете
            подключиться к серверу быстрых сообщений для мгновенного получения приходящих сообщений и других событий.
            Формат взаимодействия с LongPoll сервером
        """

        params = locals()
        method_name = self._base_method + 'getLongPollServer'
        return self._call(method_name, params)

    def isMessagesFromGroupAllowed(self, group_id: str = None, user_id: str = None) -> dict:
        """
        Возвращает информацию о том, разрешена ли отправка сообщений от сообщества пользователю.

        :param group_id: идентификатор сообщества.
        :param user_id: идентификатор пользователя.

        :return: Возвращает объект с единственным полем is_allowed (integer, [0,1]). Если отправка сообщений
            разрешена, поле содержит 1, иначе — 0
        """

        params = locals()
        method_name = self._base_method + 'isMessagesFromGroupAllowed'
        return self._call(method_name, params)

    def joinChatByInviteLink(self, link: str = None) -> dict:
        """
        Позволяет присоединиться к чату по ссылке-приглашению.

        :param link: ссылка-приглашение.

        :return: Возвращает идентификатор чата в поле chat_id
        """

        params = locals()
        method_name = self._base_method + 'joinChatByInviteLink'
        return self._call(method_name, params)

    def markAsAnsweredConversation(self, peer_id: str = None, answered: str = None, group_id: str = None) -> dict:
        """
        Помечает беседу как отвеченную либо снимает отметку.

        :param peer_id: идентификатор беседы
        :param answered: 1 - беседа отмечена отвеченной, 0 - неотвеченной
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'markAsAnsweredConversation'
        return self._call(method_name, params)

    def markAsImportant(self, message_ids: str = None, important: str = None) -> dict:
        """
        Помечает сообщения как важные либо снимает отметку.

        :param message_ids: список идентификаторов сообщений, которые необходимо пометить.
        :param important: 1, если сообщения необходимо пометить, как важные;  0, если необходимо снять пометку.

        :return: Возвращает массив идентификаторов успешно помеченных сообщений
        """

        params = locals()
        method_name = self._base_method + 'markAsImportant'
        return self._call(method_name, params)

    def markAsImportantConversation(self, peer_id: str = None, important: str = None, group_id: str = None) -> dict:
        """
        Помечает беседу как важную либо снимает отметку.

        :param peer_id: идентификатор беседы
        :param important: 1, если сообщения необходимо пометить, как важные;  0, если необходимо снять пометку.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'markAsImportantConversation'
        return self._call(method_name, params)

    def markAsRead(self, message_ids: str = None, peer_id: str = None, start_message_id: str = None,
                   group_id: str = None) -> dict:
        """
        Помечает сообщения как прочитанные.

        :param message_ids: идентификаторы сообщений.
        :param peer_id: идентификатор назначения.  Для пользователя:  id пользователя.   Для групповой беседы:
            2000000000 + id беседы.   Для сообщества:  -id сообщества.
        :param start_message_id: при передаче этого параметра будут помечены как прочитанные все сообщения,
            начиная с данного.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'markAsRead'
        return self._call(method_name, params)

    def pin(self, peer_id: str = None, message_id: str = None) -> dict:
        """
        Закрепляет сообщение.

        :param peer_id: идентификатор назначения.  Для пользователя:  id пользователя.   Для групповой беседы:
            2000000000 + id беседы.   Для сообщества:  -id сообщества.
        :param message_id: идентификатор сообщения, которое нужно закрепить.

        :return: Возвращает объект закрепленного сообщения
        """

        params = locals()
        method_name = self._base_method + 'pin'
        return self._call(method_name, params)

    def removeChatUser(self, chat_id: str = None, user_id: str = None, member_id: str = None) -> dict:
        """
        Исключает из мультидиалога пользователя, если текущий пользователь или сообщество является администратором
        беседы либо текущий пользователь пригласил исключаемого пользователя.

        :param chat_id: идентификатор беседы.
        :param user_id: идентификатор пользователя, которого необходимо исключить из беседы. Может быть меньше
            нуля в случае, если пользователь приглашён по email.
        :param member_id: идентификатор участника, которого необходимо исключить из беседы. Для сообществ —
            идентификатор сообщества со знаком «минус».

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeChatUser'
        return self._call(method_name, params)

    def restore(self, message_id: str = None, group_id: str = None) -> dict:
        """
        Восстанавливает удаленное сообщение.

        :param message_id: идентификатор сообщения, которое нужно восстановить.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'restore'
        return self._call(method_name, params)

    def search(self, q: str = None, peer_id: str = None, date: str = None, preview_length: str = None,
               offset: str = None, count: str = None, extended: str = None, fields: str = None,
               group_id: str = None) -> dict:
        """
        Возвращает список найденных личных сообщений текущего пользователя по введенной строке поиска.

        :param q: под, по которой будет производиться поиск.
        :param peer_id: фильтр по идентификатору назначения для поиска по отдельному диалогу.  Для пользователя:
            id пользователя.   Для групповой беседы:  2000000000 + id беседы.   Для сообщества:  -id сообщества.
        :param date: дата в формате DDMMYYYY — если параметр задан, в ответе будут только сообщения, отправленные
            до указанной даты.
        :param preview_length: количество символов, по которому нужно обрезать сообщение. Укажите 0, если Вы не
            хотите обрезать сообщение. (по умолчанию сообщения не обрезаются).
        :param offset: смещение, необходимое для выборки определенного подмножества сообщений из списка
            найденных.
        :param count: количество сообщений, которое необходимо получить.
        :param extended: 1 — возвращать дополнительные поля для пользователей и сообществ. В ответе будет
            содержаться массив объектов бесед.
        :param fields: список дополнительных полей для пользователей и сообществ.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов, описывающих личные сообщения, в поле items.  Обратите внимание, даже при использовании
            параметра offset максимальное число доступных результатов — 10000.  Если extended = 1, в поле items
            возвращается массив объектов бесед. Дополнительно возвращаются массивы profiles и groups, содержащие
            объекты пользователей и сообществ
        """

        params = locals()
        method_name = self._base_method + 'search'
        return self._call(method_name, params)

    def searchConversations(self, q: str = None, count: str = None, extended: str = None, fields: str = None,
                            group_id: str = None) -> dict:
        """
        Позволяет искать диалоги.

        :param q: поисковой запрос.
        :param count: максимальное число результатов для получения.
        :param extended: 1 — возвращать дополнительные поля.
        :param fields: дополнительные поля пользователей и сообществ, которые необходимо вернуть в ответе.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: Возвращает общее число результатов в поле count (integer) и массив объектов диалогов в поле
            items
        """

        params = locals()
        method_name = self._base_method + 'searchConversations'
        return self._call(method_name, params)

    def send(self, user_id: str = None, random_id: str = None, peer_id: str = None, domain: str = None,
             chat_id: str = None, user_ids: str = None, message: str = None, lat: str = None, long: str = None,
             attachment: str = None, reply_to: str = None, forward_messages: str = None, sticker_id: str = None,
             group_id: str = None, keyboard: str = None, payload: str = None, dont_parse_links: str = None,
             disable_mentions: str = None, intent: str = None) -> dict:
        """
        Отправляет сообщение.

        :param user_id: идентификатор пользователя, которому отправляется сообщение.
        :param random_id: уникальный (в привязке к API_ID и ID отправителя) идентификатор, предназначенный для
            предотвращения повторной отправки одинакового сообщения. Сохраняется вместе с сообщением и доступен в
            истории сообщений.  Заданный random_id используется для проверки уникальности за всю историю сообщений,
            поэтому используйте большой диапазон (до int64).
        :param peer_id: идентификатор назначения.  Для пользователя:  id пользователя.   Для групповой беседы:
            2000000000 + id беседы.   Для сообщества:  -id сообщества.
        :param domain: короткий адрес пользователя (например, illarionov).
        :param chat_id: идентификатор беседы, к которой будет относиться сообщение.
        :param user_ids: идентификаторы получателей сообщения (при необходимости отправить сообщение сразу
            нескольким пользователям). Доступно только для ключа доступа сообщества. Максимальное количество
            идентификаторов: 100.
        :param message: текст личного сообщения. Обязательный параметр, если не задан параметр attachment.
        :param lat: географическая широта (от -90 до 90).
        :param long: географическая долгота (от -180 до 180).
        :param attachment: медиавложения к личному сообщению, перечисленные через запятую. Каждое прикрепление
            представлено в формате: <type><owner_id>_<media_id>  <type> — тип медиавложения:   photo — фотография;
            video — видеозапись;  audio — аудиозапись;  doc — документ;  wall — запись на стене;  market — товар.
            poll — опрос.     <owner_id> — идентификатор владельца медиавложения (обратите внимание, если объект
            находится в сообществе, этот параметр должен быть отрицательным). <media_id> — идентификатор
            медиавложения.  Например: photo100172_166443618  Параметр является обязательным, если не задан параметр
            message.  В случае, если прикрепляется объект, принадлежащий другому пользователю следует добавлять к
            вложению его access_key в формате <type><owner_id>_<media_id>_<access_key>, Например:
            video85635407_165186811_69dff3de4372ae9b6e
        :param reply_to: идентификатор сообщения, на которое требуется ответить.
        :param forward_messages: идентификаторы пересылаемых сообщений, перечисленные через запятую.
            Перечисленные сообщения отправителя будут отображаться в теле письма у получателя. Не более 100 значений
            на верхнем уровне, максимальный уровень вложенности: 45, максимальное количество пересылаемых сообщений
            500  Например: 123,431,544
        :param sticker_id: идентификатор стикера.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).
        :param keyboard: объект, описывающий клавиатуру бота.
        :param payload: Полезная нагрузка
        :param dont_parse_links: 1 — не создавать сниппет ссылки из сообщения
        :param disable_mentions: 1 - отключить уведомление об упоминании в сообщении
        :param intent: , описывающая интенты

        :return: После успешного выполнения возвращает идентификатор отправленного сообщения.  Если передан
            параметр user_ids, возвращает массив объектов, каждый из которых содержит поля:   peer_id — идентификатор
            назначения;  message_id — идентификатор сообщения;  error — сообщение об ошибке, если сообщение не было
            доставлено получателю
        """

        params = locals()
        method_name = self._base_method + 'send'
        return self._call(method_name, params)

    def setActivity(self, user_id: str = None, type: str = None, peer_id: str = None, group_id: str = None) -> dict:
        """
        Изменяет статус набора текста пользователем в диалоге.

        :param user_id: идентификатор пользователя.
        :param type: typing — пользователь начал набирать текст,  audiomessage — пользователь записывает
            голосовое сообщение
        :param peer_id: идентификатор назначения.  Для групповой беседы:  2000000000 + id беседы.   Для
            сообщества:  -id сообщества.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает 1.  Текст «N набирает сообщение...» отображается в течение
            10 секунд после вызова метода, либо до момента отправки сообщения
        """

        params = locals()
        method_name = self._base_method + 'setActivity'
        return self._call(method_name, params)

    def setChatPhoto(self, file: str = None) -> dict:
        """
        Позволяет установить фотографию мультидиалога, загруженную с помощью метода photos.getChatUploadServer.

        :param file: Содержимое поля response из ответа специального upload сервера, полученного в результате
            загрузки изображения на адрес, полученный методом photos.getChatUploadServer.

        :return: После успешного выполнения возвращает объект, содержащий следующие поля:  message_id
            — идентификатор отправленного системного сообщения;  chat — объект мультидиалога
        """

        params = locals()
        method_name = self._base_method + 'setChatPhoto'
        return self._call(method_name, params)

    def unpin(self, peer_id: str = None, group_id: str = None) -> dict:
        """
        Открепляет сообщение.

        :param peer_id: идентификатор назначения.  Для пользователя:  id пользователя.   Для групповой беседы:
            2000000000 + id беседы.   Для сообщества:  -id сообщества.
        :param group_id: идентификатор сообщества (для сообщений сообщества с ключом доступа пользователя).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'unpin'
        return self._call(method_name, params)



from ApiMethod import ApiMethod


class Orders(ApiMethod):
    def __init__(self, access_token, v, session, calls_per_second):
        super(Orders, self).__init__(access_token, v, session, calls_per_second)
        self._base_method = 'orders.'

    def cancelSubscription(self, user_id: str = None, subscription_id: str = None, pending_cancel: str = None) -> dict:
        """
        Отменяет подписку.

        :param user_id: идентификатор пользователя.
        :param subscription_id: идентификатор подписки.
        :param pending_cancel: 1 — отключить подписку по истечении текущего оплаченного периода;  0 — отключить
            подписку сразу.

        :return: После успешного выполнения возвращает 1. При отмене подписки на адрес обратного вызова будет
            отправлено платёжное уведомление с типом subscription_status_change
        """

        params = locals()
        method_name = self._base_method + 'cancelSubscription'
        return self._call(method_name, params)

    def changeState(self, order_id: str = None, action: str = None, app_order_id: str = None,
                    test_mode: str = None) -> dict:
        """
        Изменяет состояние заказа.

        :param order_id: идентификатор заказа.
        :param action: действие, которое нужно произвести с заказом.  Возможные действия:   cancel — отменить
            неподтверждённый заказ.  charge — подтвердить неподтверждённый заказ. Применяется только если не удалось
            обработать уведомление order_change_state.  refund — отменить подтверждённый заказ.
        :param app_order_id: внутренний идентификатор заказа в приложении.
        :param test_mode: если этот параметр равен 1, изменяется состояние заказа тестового режима. По умолчанию
            0.

        :return: После успешного выполнения возвращает новый статус заказа.   Статусы заказа   chargeable —
            неподтверждённый заказ. В это состояние заказы попадают в случае, если магазин не обрабатывает
            уведомления.  declined — отменённый заказ на этапе получения информации о товаре, например, была получена
            ошибка 20, "Товара не существует". В это состояние заказ может попасть из состояния chargeable.
            cancelled — отменённый заказ. В это состояние заказ может попасть из состояния chargeable.  charged —
            оплаченный заказ. В это состояние заказ может попасть из состояния chargeable, либо сразу после оплаты,
            если приложение обрабатывает уведомления.  refunded — отменённый после оплаты заказ, голоса возвращены
            пользователю
        """

        params = locals()
        method_name = self._base_method + 'changeState'
        return self._call(method_name, params)

    def get(self, offset: str = None, count: str = None, test_mode: str = None) -> dict:
        """
        Возвращает список заказов.

        :param offset: смещение относительно самого нового найденного заказа для выборки определенного
            подмножества. По умолчанию 0.
        :param count: количество возвращаемых заказов.
        :param test_mode: если этот параметр равен 1, возвращается список заказов тестового режима. По умолчанию
            0.

        :return: После успешного выполнения возвращает массив найденных заказов, отсортированный по дате в
            обратном порядке (самый новый в начале).   Статусы заказа   chargeable — неподтверждённый заказ. В это
            состояние заказы попадают в случае, если магазин не обрабатывает уведомления.  declined — отменённый
            заказ на этапе получения информации о товаре, например, была получена ошибка 20, "Товара не существует".
            В это состояние заказ может попасть из состояния chargeable.  cancelled — отменённый заказ. В это
            состояние заказ может попасть из состояния chargeable.  charged — оплаченный заказ. В это состояние заказ
            может попасть из состояния chargeable, либо сразу после оплаты, если приложение обрабатывает уведомления.
            refunded — отменённый после оплаты заказ, голоса возвращены пользователю
        """

        params = locals()
        method_name = self._base_method + 'get'
        return self._call(method_name, params)

    def getAmount(self, user_id: str = None, votes: str = None) -> dict:
        """
        Возвращает стоимость голосов в валюте пользователя.

        :param user_id: идентификатор пользователя
        :param votes: список голосов. Например: 1,7,77

        :return: Возвращает валюту пользователя и массив результатов для каждого значения из votes
        """

        params = locals()
        method_name = self._base_method + 'getAmount'
        return self._call(method_name, params)

    def getById(self, order_id: str = None, order_ids: str = None, test_mode: str = None) -> dict:
        """
        Возвращает информацию об отдельном заказе.

        :param order_id: идентификатор заказа.
        :param order_ids: идентификаторы заказов (при запросе информации о нескольких заказах).
        :param test_mode: если этот параметр равен 1, возвращаются заказы тестового режима. По умолчанию 0.

        :return: Возвращается массив найденных заказов, отсортированный по дате в обратном порядке (самый новый в
            начале).   Статусы заказа   chargeable — неподтверждённый заказ. В это состояние заказы попадают в
            случае, если магазин не обрабатывает уведомления.  declined — отменённый заказ на этапе получения
            информации о товаре, например, была получена ошибка 20, "Товара не существует". В это состояние заказ
            может попасть из состояния chargeable.  cancelled — отменённый заказ. В это состояние заказ может попасть
            из состояния chargeable.  charged — оплаченный заказ. В это состояние заказ может попасть из состояния
            chargeable, либо сразу после оплаты, если приложение обрабатывает уведомления.  refunded — отменённый
            после оплаты заказ, голоса возвращены пользователю
        """

        params = locals()
        method_name = self._base_method + 'getById'
        return self._call(method_name, params)

    def getUserSubscriptionById(self, user_id: str = None, subscription_id: str = None) -> dict:
        """
        Получает информацию о подписке по её идентификатору.

        :param user_id: идентификатор пользователя.
        :param subscription_id: идентификатор подписки.

        :return: Возвращает объект, описывающий подписку. Содержит следующие поля:   id (integer) — идентификатор
            подписки.  item_id (string) — идентификатор товара в приложении.  status (string) — статус подписки.
            Возможные значения:   chargeable — неподтвержденная подписка;  active — подписка активна;  cancelled —
            подписка отменена.   price (integer) — стоимость подписки.  period (integer) — период подписки.
            create_time (integer) — дата создания в Unixtime.  update_time (integer) — дата обновления в Unixtime.
            period_start_time (integer) — дата начала периода в Unixtime.  next_bill_time (integer) — дата следующего
            платежа в Unixtime (если status = active).  trial_expire_time (integer) — дата истечения триал-периода
            (если есть).  pending_cancel (boolean, [true]) — true, если подписка ожидает отмены.  cancel_reason
            (string) — причина отмены (если есть). Возможные значения:   user_decision — по инициативе пользователя;
            app_decision — по инициативе приложения;  payment_fail — из-за проблемы с платежом;  unknown — причина
            неизвестна.   test_mode (boolean, [true]) — true, если используется тестовый режим
        """

        params = locals()
        method_name = self._base_method + 'getUserSubscriptionById'
        return self._call(method_name, params)

    def getUserSubscriptions(self, user_id: str = None) -> dict:
        """
        Получает список активных подписок пользователя.

        :param user_id: идентификатор пользователя, подписки которого необходимо получить.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов, описывающих подписку, в поле items. Каждый объект массива items содержит следующие поля:    id
            (integer) — идентификатор подписки.  item_id (string) — идентификатор товара в приложении.  status
            (string) — статус подписки. Возможные значения:   active — подписка активна.   price (integer) —
            стоимость подписки.  period (integer) — период подписки.  create_time (integer) — дата создания в
            Unixtime.  update_time (integer) — дата обновления в Unixtime.  period_start_time (integer) — дата начала
            периода в Unixtime.  next_bill_time (integer) — дата следующего платежа в Unixtime (если status =
            active).  trial_expire_time (integer) — дата истечения триал-периода (если есть).  pending_cancel
            (boolean, [true]) — true, если подписка ожидает отмены.  cancel_reason (string) — причина отмены (если
            есть). Возможные значения:   user_decision — по инициативе пользователя;  app_decision — по инициативе
            приложения;  payment_fail — из-за проблемы с платежом;  unknown — причина неизвестна.   test_mode
            (boolean, [true]) — true, если используется тестовый режим
        """

        params = locals()
        method_name = self._base_method + 'getUserSubscriptions'
        return self._call(method_name, params)

    def updateSubscription(self, user_id: str = None, subscription_id: str = None, price: str = None) -> dict:
        """
        Обновляет цену подписки для пользователя.

        :param user_id: идентификатор пользователя.
        :param subscription_id: идентификатор подписки. Подписка должна быть активна.
        :param price: новая стоимость подписки (должна быть ниже, чем текущая).

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'updateSubscription'
        return self._call(method_name, params)




from ApiMethod import ApiMethod


class Fave(ApiMethod):
    def __init__(self, access_token, v, session, calls_per_second):
        super(Fave, self).__init__(access_token, v, session, calls_per_second)
        self._base_method = 'fave.'

    def addArticle(self, url: str = None) -> dict:
        """
        Добавляет статью в закладки.

        :param url: ссылка на добавляемую статью.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addArticle'
        return self._call(method_name, params)

    def addLink(self, link: str = None) -> dict:
        """
        Добавляет ссылку в закладки.

        :param link: адрес добавляемой ссылки. Поддерживаются только внутренние ссылки на vk.com.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addLink'
        return self._call(method_name, params)

    def addPage(self, user_id: str = None, group_id: str = None) -> dict:
        """
        Добавляет сообщество или пользователя в закладки.

        :param user_id: идентификатор пользователя, которого нужно добавить в закладки.
        :param group_id: идентификатор сообщества, которое нужно добавить в закладки.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addPage'
        return self._call(method_name, params)

    def addPost(self, owner_id: str = None, id: str = None, access_key: str = None, ref: str = None,
                track_code: str = None, source: str = None) -> dict:
        """
        Добавляет запись со стены пользователя или сообщества в закладки.

        :param owner_id: идентификатор пользователя или сообщества, чью запись со стены требуется добавить в
            закладки. Обратите внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком
            "-" — например, owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param id: идентификатор записи, которую необходимо добавить в закладки.
        :param access_key: специальный код доступа для приватных постов.
        :param ref:
        :param track_code:
        :param source:

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addPost'
        return self._call(method_name, params)

    def addProduct(self, owner_id: str = None, id: str = None, access_key: str = None) -> dict:
        """
        Добавляет товар в закладки.

        :param owner_id: идентификатор пользователя или сообщества, чей товар требуется добавить в закладки.
            Обратите внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" —
            например, owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param id: идентификатор товара.
        :param access_key: специальный код доступа для приватных товаров.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addProduct'
        return self._call(method_name, params)

    def addTag(self, name: str = None, position: str = None) -> dict:
        """
        Создает метку закладок.

        :param name: название метки.
        :param position: положение метки.  Возможные значения:   front - начало списка,  back - конец списка.

        :return: В случае успешного выполнения возвращает объект метки с полями name - названием метки и id -
            идентификатором созданной метки
        """

        params = locals()
        method_name = self._base_method + 'addTag'
        return self._call(method_name, params)

    def addVideo(self, owner_id: str = None, id: str = None, access_key: str = None, ref: str = None) -> dict:
        """
        Добавляет видеозапись в закладки.

        :param owner_id: идентификатор пользователя или сообщества, владельца видеозаписи, которую требуется
            добавить в закладки. Обратите внимание, идентификатор сообщества в параметре owner_id необходимо
            указывать со знаком "-" — например, owner_id=-1 соответствует идентификатору сообщества ВКонтакте API
            (club1)
        :param id: идентификатор видеозаписи.
        :param access_key: специальный код доступа для приватных видеозаписей.
        :param ref:

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addVideo'
        return self._call(method_name, params)

    def editTag(self, id: str = None, name: str = None) -> dict:
        """
        Редактирует метку.

        :param id: идентификатор метки.
        :param name: новое название метки.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'editTag'
        return self._call(method_name, params)

    def get(self, extended: str = None, item_type: str = None, tag_id: str = None, offset: str = None,
            count: str = None, fields: str = None, is_from_snackbar: str = None) -> dict:
        """
        Возвращает объекты, добавленные в закладки пользователя.

        :param extended: 1, если необходимо получить информацию о пользователе. По умолчанию: 0.
        :param item_type: Типы объектов, которые необходимо вернуть. Возможные значения:   post — записи;  video
            — видеозаписи;  product — товары;  article — статьи;  link — ссылки.
        :param tag_id: идентификатор метки, закладки отмеченные которой требуется вернуть.
        :param offset: смещение относительно первого объекта в закладках пользователя для выборки определенного
            подмножества.
        :param count: количество возвращаемых закладок.
        :param fields: список дополнительных полей профилей, которые необходимо вернуть. См. подробное описание.
        :param is_from_snackbar:

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов, добавленных в закладки, в поле items.  Каждый из объектов содержит следующие поля:  type
            stringтип объекта, добавленного в закладки. Возможные значения:   post — запись на стене;  video —
            видеозапись;  product — товар;  article — статья;  link — ссылки. seen  booleanявляется ли закладка
            просмотренной. added_date  integerдата добавления объекта в закладки в формате Unixtime. tags
            arrayмассив меток объекта в списке закладок. {object_type}  objectобъект, добавленный в закладки.  Если
            был задан параметр extended=1, возвращает число результатов в поле count, отдельно массив объектов
            пользователей в поле profiles и сообществ в поле groups
        """

        params = locals()
        method_name = self._base_method + 'get'
        return self._call(method_name, params)

    def getPages(self, offset: str = None, count: str = None, type: str = None, fields: str = None,
                 tag_id: str = None) -> dict:
        """
        Возвращает страницы пользователей и сообществ, добавленных в закладки.

        :param offset: смещение относительно первого объекта в закладках пользователя для выборки определенного
            подмножества.
        :param count: количество возвращаемых закладок.
        :param type: Типы объектов, которые необходимо вернуть. Возможные значения:   users — вернуть только
            пользователей;  groups — вернуть только сообщества;  hints — топ сообществ и пользователей.   Если
            параметр не указан — вернутся объекты пользователей и сообществ, добавленных в закладки, в порядке
            добавления.
        :param fields: список дополнительных полей для объектов user и group, которые необходимо вернуть.
        :param tag_id: идентификатор метки, закладки отмеченные которой требуется вернуть.

        :return
        """

        params = locals()
        method_name = self._base_method + 'getPages'
        return self._call(method_name, params)

    def getTags(self) -> dict:
        """
        Возвращает список меток в закладках.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов меток в поле items.  Поля, описывающие объект метки: id - идентификатор метки и name - название
        """

        params = locals()
        method_name = self._base_method + 'getTags'
        return self._call(method_name, params)

    def markSeen(self) -> dict:
        """
        Отмечает закладки как просмотренные.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'markSeen'
        return self._call(method_name, params)

    def removeArticle(self, owner_id: str = None, article_id: str = None, ref: str = None) -> dict:
        """
        Удаляет статью из закладок.

        :param owner_id: идентификатор владельца статьи.
        :param article_id: идентификатор статьи.
        :param ref:

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeArticle'
        return self._call(method_name, params)

    def removeLink(self, link_id: str = None) -> dict:
        """
        Удаляет ссылку из списка закладок пользователя.

        :param link_id: идентификатор ссылки.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeLink'
        return self._call(method_name, params)

    def removePage(self, user_id: str = None, group_id: str = None) -> dict:
        """
        Удаляет из закладок сообщество или страницу пользователя.

        :param user_id: идентификатор пользователя, которого следует удалить из закладок.
        :param group_id: идентификатор сообщества, которое следует удалить из закладок.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removePage'
        return self._call(method_name, params)

    def removePost(self, owner_id: str = None, id: str = None) -> dict:
        """
        Удаляет из закладок запись на стене пользователя или сообщества.

        :param owner_id: идентификатор владельца записи. Обратите внимание, идентификатор сообщества в параметре
            owner_id необходимо указывать со знаком "-" — например, owner_id=-1 соответствует идентификатору
            сообщества ВКонтакте API (club1)
        :param id: идентификатор записи на стене.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removePost'
        return self._call(method_name, params)

    def removeProduct(self, owner_id: str = None, id: str = None) -> dict:
        """
        Удаляет товар из закладок.

        :param owner_id: идентификатор пользователя или сообщества, чей товар требуется удалить из закладок.
            Обратите внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" —
            например, owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param id: идентификатор товара.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeProduct'
        return self._call(method_name, params)

    def removeTag(self, id: str = None) -> dict:
        """
        Удаляет метку закладок.

        :param id: идентификатор метки.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeTag'
        return self._call(method_name, params)

    def removeVideo(self, owner_id: str = None, id: str = None) -> dict:
        """
        Удаляет видеозапись из списка закладок.

        :param owner_id: идентификатор пользователя или сообщества, владельца видеозаписи, которую требуется
            удалить из закладок. Обратите внимание, идентификатор сообщества в параметре owner_id необходимо
            указывать со знаком "-" — например, owner_id=-1 соответствует идентификатору сообщества ВКонтакте API
            (club1)
        :param id: идентификатор видеозаписи.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeVideo'
        return self._call(method_name, params)

    def reorderTags(self, ids: str = None) -> dict:
        """
        Меняет порядок меток закладок в списке.

        :param ids: перечисленные через запятую все идентификаторы меток пользователя в том порядке, в котором их
            требуется отображать.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reorderTags'
        return self._call(method_name, params)

    def setPageTags(self, user_id: str = None, group_id: str = None, tag_ids: str = None) -> dict:
        """
        Устанавливает метку странице пользователя или сообщества.

        :param user_id: идентификатор пользователя, которому требуется проставить метку в закладках. Обязательный
            параметр, если не задан параметр group_id.
        :param group_id: идентификатор сообщества, которому требуется проставить метку в закладках. Обязательный
            параметр, если не задан параметр user_id.
        :param tag_ids: перечисленные через запятую идентификаторы тегов, которые требуется присвоить странице.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'setPageTags'
        return self._call(method_name, params)

    def setTags(self, item_type: str = None, item_owner_id: str = None, item_id: str = None, tag_ids: str = None,
                link_id: str = None, link_url: str = None) -> dict:
        """
        Устанавливает метку выбранному объекту в списке закладок.

        :param item_type: Тип объекта, которому необходимо присвоить метку. Возможные значения:   post — запись
            на стене;  video — видеозапись;  product — товар;  article — статья;  link — ссылка.   Для работы с
            объектами пользователя или сообщества используйте метод fave.setPageTags
        :param item_owner_id: идентификатор владельца объекта, которому требуется присвоить метку. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param item_id: идентификатор объекта.
        :param tag_ids: идентификатор метки, которую требуется присвоить объекту.
        :param link_id: идентификатор ссылки, которой требуется присвоить метку.
        :param link_url:

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'setTags'
        return self._call(method_name, params)

    def trackPageInteraction(self, user_id: str = None, group_id: str = None) -> dict:
        """
        Устанавливает страницу пользователя или сообщества в топ закладок.

        :param user_id: идентификатор пользователя, которому требуется проставить метку в закладках. Обязательный
            параметр, если не задан параметр group_id.
        :param group_id: идентификатор сообщества, которому требуется проставить метку в закладках. Обязательный
            параметр, если не задан параметр user_id.

        :return: В случае успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'trackPageInteraction'
        return self._call(method_name, params)



from ApiMethod import ApiMethod


class Video(ApiMethod):
    def __init__(self, access_token, v, session, calls_per_second):
        super(Video, self).__init__(access_token, v, session, calls_per_second)
        self._base_method = 'video.'

    def add(self, target_id: str = None, video_id: str = None, owner_id: str = None) -> dict:
        """
        Добавляет видеозапись в список пользователя.

        :param target_id: идентификатор пользователя или сообщества, в которое нужно добавить видео.  Обратите
            внимание, идентификатор сообщества в параметре target_id необходимо указывать со знаком "-" — например,
            target_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.
        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'add'
        return self._call(method_name, params)

    def addAlbum(self, group_id: str = None, title: str = None, privacy: str = None) -> dict:
        """
        Создает пустой альбом видеозаписей.

        :param group_id: идентификатор сообщества (если необходимо создать альбом в сообществе).
        :param title: название альбома.
        :param privacy: настройки доступа к альбому в специальном формате.  Приватность доступна для альбомов с
            видео в профиле пользователя.

        :return: После успешного выполнения возвращает идентификатор созданного альбома в поле album_id
        """

        params = locals()
        method_name = self._base_method + 'addAlbum'
        return self._call(method_name, params)

    def addToAlbum(self, target_id: str = None, album_id: str = None, album_ids: str = None, owner_id: str = None,
                   video_id: str = None) -> dict:
        """
        Позволяет добавить видеозапись в альбом.

        :param target_id: идентификатор владельца альбома, в который нужно добавить видео.  Обратите внимание,
            идентификатор сообщества в параметре target_id необходимо указывать со знаком "-" — например,
            target_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома, в который нужно добавить видео.  Для добавления видео в общий
            альбом «Добавленные» передавайте -2.
        :param album_ids: идентификаторы альбомов, в которые нужно добавить видео.
        :param owner_id: идентификатор владельца видеозаписи.  Обратите внимание, идентификатор сообщества в
            параметре owner_id необходимо указывать со знаком "-" — например, owner_id=-1 соответствует
            идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'addToAlbum'
        return self._call(method_name, params)

    def createComment(self, owner_id: str = None, video_id: str = None, message: str = None, attachments: str = None,
                      from_group: str = None, reply_to_comment: str = None, sticker_id: str = None,
                      guid: str = None) -> dict:
        """
        Cоздает новый комментарий к видеозаписи

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.
        :param message: текст комментария. Обязательный параметр, если не задан параметр attachments.
        :param attachments: список объектов, приложенных к комментарию и разделённых символом ",". Поле
            attachments представляется в формате: <type><owner_id>_<media_id>,<type><owner_id>_<media_id> <type> —
            тип медиа-вложения: photo — фотография  video — видеозапись  audio — аудиозапись  doc — документ
            <owner_id> — идентификатор владельца медиа-вложения  <media_id> — идентификатор медиа-вложения.
            Например: photo100172_166443618,photo66748_265827614 Параметр является обязательным, если не задан
            параметр message.
        :param from_group: этот параметр учитывается, если owner_id < 0 (комментарий к видеозаписи группы). 1 —
            комментарий будет опубликован от имени группы, 0 — комментарий будет опубликован от имени пользователя.
            по умолчанию: 0.
        :param reply_to_comment: идентификатор комментария, в ответ на который должен быть добавлен новый
            комментарий.
        :param sticker_id: идентификатор стикера.
        :param guid: уникальный идентификатор, предназначенный для предотвращения повторной отправки одинакового
            комментария.

        :return: После успешного выполнения возвращает идентификатор созданного комментария
        """

        params = locals()
        method_name = self._base_method + 'createComment'
        return self._call(method_name, params)

    def delete(self, video_id: str = None, owner_id: str = None, target_id: str = None) -> dict:
        """
        Удаляет видеозапись со страницы пользователя.

        :param video_id: идентификатор видеозаписи.
        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param target_id: идентификатор пользователя или сообщества, для которого нужно удалить видеозапись.
            Обратите внимание, идентификатор сообщества в параметре target_id необходимо указывать со знаком "-" —
            например, target_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'delete'
        return self._call(method_name, params)

    def deleteAlbum(self, group_id: str = None, album_id: str = None) -> dict:
        """
        Удаляет альбом видеозаписей.

        :param group_id: идентификатор сообщества (если альбом, который необходимо удалить, принадлежит
            сообществу).
        :param album_id: идентификатор альбома.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'deleteAlbum'
        return self._call(method_name, params)

    def deleteComment(self, owner_id: str = None, comment_id: str = None) -> dict:
        """
        Удаляет комментарий к видеозаписи.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись.
        :param comment_id: идентификатор комментария.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'deleteComment'
        return self._call(method_name, params)

    def edit(self, owner_id: str = None, video_id: str = None, name: str = None, desc: str = None,
             privacy_view: str = None, privacy_comment: str = None, no_comments: str = None,
             repeat: str = None) -> dict:
        """
        Редактирует данные видеозаписи.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.
        :param name: новое название для видеозаписи.
        :param desc: новое описание для видеозаписи.
        :param privacy_view: настройки приватности просмотра видеозаписи в специальном формате. Приватность
            доступна для видеозаписей, которые пользователь загрузил в профиль.
        :param privacy_comment: настройки приватности комментирования видеозаписи в специальном формате.
            Приватность доступна для видеозаписей, которые пользователь загрузил в профиль.
        :param no_comments: закрыть комментарии (для видео из сообществ).
        :param repeat: зацикливание воспроизведения видеозаписи.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'edit'
        return self._call(method_name, params)

    def editAlbum(self, group_id: str = None, album_id: str = None, title: str = None, privacy: str = None) -> dict:
        """
        Редактирует альбом с видео.

        :param group_id: идентификатор сообщества (если нужно отредактировать альбом, принадлежащий сообществу).
        :param album_id: идентификатор альбома.
        :param title: новое название для альбома.
        :param privacy: уровень доступа к альбому в специальном формате.  Приватность доступна для альбомов с
            видео в профиле пользователя.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'editAlbum'
        return self._call(method_name, params)

    def editComment(self, owner_id: str = None, comment_id: str = None, message: str = None,
                    attachments: str = None) -> dict:
        """
        Изменяет текст комментария к видеозаписи.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param comment_id: идентификатор комментария.
        :param message: новый текст комментария. Обязательный параметр, если не задан параметр attachments.
        :param attachments: новый список объектов, приложенных к комментарию и разделённых символом ",". Поле
            attachments представляется в формате: <type><owner_id>_<media_id>,<type><owner_id>_<media_id> <type> —
            тип медиа-вложения: photo — фотография  video — видеозапись  audio — аудиозапись  doc — документ
            <owner_id> — идентификатор владельца медиа-вложения  <media_id> — идентификатор медиа-вложения.
            Например: photo100172_166443618,photo66748_265827614 Обязательный параметр, если не задан параметр
            message.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'editComment'
        return self._call(method_name, params)

    def get(self, owner_id: str = None, videos: str = None, album_id: str = None, count: str = None, offset: str = None,
            extended: str = None) -> dict:
        """
        Возвращает информацию о видеозаписях.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежат видеозаписи. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param videos: перечисленные через запятую идентификаторы — идущие через знак подчеркивания id
            пользователей, которым принадлежат видеозаписи, и id самих видеозаписей. Если видеозапись принадлежит
            сообществу, то в качестве первого параметра используется -id сообщества. Пример значения videos:
            -4363_136089719,13245770_137352259  Некоторые видеозаписи, идентификаторы которых могут быть получены
            через API, закрыты приватностью, и не будут получены. В этом случае следует использовать ключ доступа
            access_key в её идентификаторе. Пример значения videos:   1_129207899_220df2876123d3542f,
            6492_135055734_e0a9bcc31144f67fbd  Поле access_key будет возвращено вместе с остальными данными
            видеозаписи в методах, которые возвращают видеозаписи, закрытые приватностью, но доступные в данном
            контексте. Например, данное поле имеют видеозаписи, возвращаемые методом messages.get.
        :param album_id: идентификатор альбома, видеозаписи из которого нужно вернуть.
        :param count: количество возвращаемых видеозаписей.
        :param offset: смещение относительно первой найденной видеозаписи для выборки определенного подмножества.
        :param extended: определяет, возвращать ли информацию о настройках приватности видео для текущего
            пользователя.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов видеозаписей с дополнительным полем comments, содержащим число комментариев у видеозаписи, в
            поле items.  Если был задан параметр extended=1, возвращаются дополнительные поля:   privacy_view —
            настройки приватности в формате настроек приватности; (приходит только для текущего пользователя)
            privacy_comment — настройки приватности в формате настроек приватности; (приходит только для текущего
            пользователя)  can_comment — может ли текущий пользователь оставлять комментарии к ролику (1 — может, 0 —
            не может);  can_repost — может ли текущий пользователь скопировать ролик с помощью функции «Рассказать
            друзьям» (1 — может, 0 — не может);  likes — информация об отметках «Мне нравится»:   user_likes — есть
            ли отметка «Мне нравится» от текущего пользователя;  count — число отметок «Мне нравится»;   repeat —
            зацикливание воспроизведения видеозаписи (1 — зацикливается, 0 — не зацикливается).   Если в Вашем
            приложении используется прямая авторизация, возвращается дополнительное поле files, содержащее ссылку на
            файл с видео (если ролик размещен на сервере ВКонтакте) или ссылку на внешний ресурс (если ролик встроен
            с какого-либо видеохостинга)
        """

        params = locals()
        method_name = self._base_method + 'get'
        return self._call(method_name, params)

    def getAlbumById(self, owner_id: str = None, album_id: str = None) -> dict:
        """
        Позволяет получить информацию об альбоме с видео.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит альбом.
        :param album_id: идентификатор альбома.

        :return: После успешного выполнения возвращает объект, который содержит следующие поля:   id —
            идентификатор альбома;  owner_id — идентификатор владельца альбома;  title — название альбома;  count —
            число видеозаписей в альбоме;  photo_320 — url обложки альбома с шириной 320px;  photo_160 — url обложки
            альбома с шириной 160px;  updated_time — время последнего обновления в формате unixtime
        """

        params = locals()
        method_name = self._base_method + 'getAlbumById'
        return self._call(method_name, params)

    def getAlbums(self, owner_id: str = None, offset: str = None, count: str = None, extended: str = None,
                  need_system: str = None) -> dict:
        """
        Возвращает список альбомов видеозаписей пользователя или сообщества.

        :param owner_id: идентификатор владельца альбомов (пользователь или сообщество). По умолчанию —
            идентификатор текущего пользователя.
        :param offset: смещение, необходимое для выборки определенного подмножества альбомов. По умолчанию: 0.
        :param count: количество альбомов, информацию о которых нужно вернуть. По умолчанию: не больше 50,
            максимальное значение: 100.
        :param extended: 1— возвращать дополнительные поля count, photo_320, photo_160 и updated_time для каждого
            альбома. Если альбом пустой, то поля photo_320 и photo_160 возвращены не будут. По умолчанию: 0.
        :param need_system: 1 — возвращать системные альбомы.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов альбомов в поле items. Каждый из объектов, описывающих альбом, содержит следующие поля:   id —
            идентификатор альбома;  owner_id — идентификатор владельца альбома;  title — название альбома
        """

        params = locals()
        method_name = self._base_method + 'getAlbums'
        return self._call(method_name, params)

    def getAlbumsByVideo(self, target_id: str = None, owner_id: str = None, video_id: str = None,
                         extended: str = None) -> dict:
        """
        Возвращает список альбомов, в которых находится видеозапись.

        :param target_id: идентификатор пользователя или сообщества, для которого нужно получить список альбомов,
            содержащих видеозапись.  Обратите внимание, идентификатор сообщества в параметре target_id необходимо
            указывать со знаком "-" — например, target_id=-1 соответствует идентификатору сообщества ВКонтакте API
            (club1)
        :param owner_id: идентификатор владельца видеозаписи.  Обратите внимание, идентификатор сообщества в
            параметре owner_id необходимо указывать со знаком "-" — например, owner_id=-1 соответствует
            идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.
        :param extended: 1 — возвращать расширенную информацию об альбомах.

        :return: Возвращает список идентификаторов альбомов, в которых видеозапись находится у пользователя или
            сообщества с идентификатором target_id. Если был передан параметр extended=1, возвращается список
            объектов альбомов с дополнительной информацией о каждом из них
        """

        params = locals()
        method_name = self._base_method + 'getAlbumsByVideo'
        return self._call(method_name, params)

    def getComments(self, owner_id: str = None, video_id: str = None, need_likes: str = None,
                    start_comment_id: str = None, offset: str = None, count: str = None, sort: str = None,
                    extended: str = None, fields: str = None) -> dict:
        """
        Возвращает список комментариев к видеозаписи.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.
        :param need_likes: 1 — будет возвращено дополнительное поле likes. По умолчанию поле likes не
            возвращается.
        :param start_comment_id: идентификатор комментария, начиная с которого нужно вернуть список (подробности
            см. ниже).
        :param offset: смещение, необходимое для выборки определенного подмножества комментариев. По умолчанию:
            0.
        :param count: количество комментариев, информацию о которых необходимо вернуть.
        :param sort: порядок сортировки комментариев. Возможные значения:   asc — от старых к новым;  desc — от
            новых к старым.
        :param extended: 1 — в ответе будут возвращены дополнительные поля profiles и groups, содержащие
            информацию о пользователях и сообществах. По умолчанию: 0.
        :param fields:

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов комментариев в поле items. Каждый объект, описывающий комментарий, содержит следующие поля:   id
            — идентификатор комментария;  from_id — идентификатор автора комментария;  date — дата добавления
            комментария в формате unixtime;  text — текст комментария;  likes — информация об отметках «Мне нравится»
            текущего комментария (если был задан параметр need_likes), объект с полями:   count — число
            пользователей, которым понравился комментарий,  user_likes — наличие отметки «Мне нравится» от текущего
            пользователя  (1 — есть, 0 — нет),  can_like — информация о том, может ли текущий пользователь поставить
            отметку «Мне нравится»  (1 — может, 0 — не может).    Если был задан параметр extended=1, возвращает
            число результатов в поле count, отдельно массив объектов комментариев в поле items, пользователей в поле
            profiles и сообществ в поле groups.  Если был передан параметр start_comment_id, будет также возвращено
            поле real_offset – итоговое смещение данного подмножества комментариев (оно может быть отрицательным,
            если был указан отрицательный offset)
        """

        params = locals()
        method_name = self._base_method + 'getComments'
        return self._call(method_name, params)

    def removeFromAlbum(self, target_id: str = None, album_id: str = None, album_ids: str = None, owner_id: str = None,
                        video_id: str = None) -> dict:
        """
        Позволяет убрать видеозапись из альбома.

        :param target_id: идентификатор владельца альбома. Обратите внимание, идентификатор сообщества в
            параметре target_id необходимо указывать со знаком "-" — например, target_id=-1 соответствует
            идентификатору сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома, из которого нужно убрать видео.
        :param album_ids: идентификаторы альбомов, из которых нужно убрать видео.
        :param owner_id: идентификатор владельца видеозаписи. Обратите внимание, идентификатор сообщества в
            параметре owner_id необходимо указывать со знаком "-" — например, owner_id=-1 соответствует
            идентификатору сообщества ВКонтакте API (club1)
        :param video_id: идентификатор видеозаписи.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'removeFromAlbum'
        return self._call(method_name, params)

    def reorderAlbums(self, owner_id: str = None, album_id: str = None, before: str = None, after: str = None) -> dict:
        """
        Позволяет изменить порядок альбомов с видео.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит альбом. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома, который нужно переместить.
        :param before: идентификатор альбома, перед которым нужно поместить текущий.
        :param after: идентификатор альбома, после которого нужно поместить текущий.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reorderAlbums'
        return self._call(method_name, params)

    def reorderVideos(self, target_id: str = None, album_id: str = None, owner_id: str = None, video_id: str = None,
                      before_owner_id: str = None, before_video_id: str = None, after_owner_id: str = None,
                      after_video_id: str = None) -> dict:
        """
        Позволяет переместить видеозапись в альбоме.

        :param target_id: идентификатор пользователя или сообщества, в чьем альбоме нужно переместить видео.
            Обратите внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" —
            например, owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param album_id: идентификатор альбома с видеозаписью, которую нужно переместить.
        :param owner_id: идентификатор владельца видеозаписи, которую нужно переместить (пользователь или
            сообщество).
        :param video_id: идентификатор видеозаписи, которую нужно переместить.
        :param before_owner_id: идентификатор владельца видеозаписи, перед которой следует поместить текущую
            (пользователь или сообщество).
        :param before_video_id: идентификатор видеозаписи, перед которой следует поместить текущую.
        :param after_owner_id: идентификатор владельца видеозаписи, после которой следует поместить текущую
            (пользователь или сообщество).
        :param after_video_id: идентификатор видеозаписи, после которой следует поместить текущую.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reorderVideos'
        return self._call(method_name, params)

    def report(self, owner_id: str = None, video_id: str = None, reason: str = None, comment: str = None,
               search_query: str = None) -> dict:
        """
        Позволяет пожаловаться на видеозапись.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись.
        :param video_id: идентификатор видеозаписи.
        :param reason: тип жалобы. Возможные значения:   0 — это спам;  1 — детская порнография;  2 — экстремизм;
            3 — насилие;  4 — пропаганда наркотиков;  5 — материал для взрослых;  6 — оскорбление.
        :param comment: комментарий для жалобы.
        :param search_query: поисковой запрос, если видеозапись была найдена через поиск.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'report'
        return self._call(method_name, params)

    def reportComment(self, owner_id: str = None, comment_id: str = None, reason: str = None) -> dict:
        """
        Позволяет пожаловаться на комментарий к видеозаписи.

        :param owner_id: идентификатор владельца видеозаписи, к которой оставлен комментарий.
        :param comment_id: идентификатор комментария.
        :param reason: тип жалобы. Возможные значения:   0 — это спам;  1 — детская порнография;  2 — экстремизм;
            3 — насилие;  4 — пропаганда наркотиков;  5 — материал для взрослых;  6 — оскорбление.

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'reportComment'
        return self._call(method_name, params)

    def restore(self, video_id: str = None, owner_id: str = None) -> dict:
        """
        Восстанавливает удаленную видеозапись.

        :param video_id: идентификатор видеозаписи.
        :param owner_id: идентификатор владельца видеозаписи (пользователь или сообщество). Обратите внимание,
            идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например, owner_id=-1
            соответствует идентификатору сообщества ВКонтакте API (club1)

        :return: После успешного выполнения возвращает 1
        """

        params = locals()
        method_name = self._base_method + 'restore'
        return self._call(method_name, params)

    def restoreComment(self, owner_id: str = None, comment_id: str = None) -> dict:
        """
        Восстанавливает удаленный комментарий к видеозаписи.

        :param owner_id: идентификатор пользователя или сообщества, которому принадлежит видеозапись. Обратите
            внимание, идентификатор сообщества в параметре owner_id необходимо указывать со знаком "-" — например,
            owner_id=-1 соответствует идентификатору сообщества ВКонтакте API (club1)
        :param comment_id: идентификатор удаленного комментария.

        :return: После успешного выполнения возвращает 1 (0, если комментарий с таким идентификатором не является
            удаленным)
        """

        params = locals()
        method_name = self._base_method + 'restoreComment'
        return self._call(method_name, params)

    def save(self, name: str = None, description: str = None, is_private: str = None, wallpost: str = None,
             link: str = None, group_id: str = None, album_id: str = None, privacy_view: str = None,
             privacy_comment: str = None, no_comments: str = None, repeat: str = None, compression: str = None) -> dict:
        """
        Возвращает адрес сервера, необходимый для загрузки, и данные видеозаписи.

        :param name: название видеофайла.
        :param description: описание видеофайла.
        :param is_private: указывается 1, если видео загружается для отправки личным сообщением. После загрузки с
            этим параметром видеозапись не будет отображаться в списке видеозаписей пользователя и не будет доступна
            другим пользователям по ее идентификатору. По умолчанию: 0.
        :param wallpost: требуется ли после сохранения опубликовать запись с видео на стене (1 — требуется, 0 —
            не требуется).  Обратите внимание, для публикации записи на стене приложение должно иметь права wall.
        :param link: url для встраивания видео с внешнего сайта, например, с Youtube. В этом случае нужно вызвать
            полученный upload_url, не прикрепляя файл, достаточно просто обратиться по этому адресу.
        :param group_id: идентификатор сообщества, в которое будет сохранен видеофайл. По умолчанию файл
            сохраняется на страницу текущего пользователя.
        :param album_id: идентификатор альбома, в который будет загружен видео файл.
        :param privacy_view: настройки приватности просмотра видеозаписи в специальном формате. Приватность
            доступна для видеозаписей, которые пользователь загрузил в профиль.
        :param privacy_comment: настройки приватности комментирования видеозаписи в специальном формате.
            Приватность доступна для видеозаписей, которые пользователь загрузил в профиль.
        :param no_comments: 1 — закрыть комментарии (для видео из сообществ). По умолчанию: 0.
        :param repeat: зацикливание воспроизведения видеозаписи.
        :param compression:

        :return: Возвращает объект, который имеет поля upload_url, video_id, title, description, owner_id.  Метод
            может быть вызван не более 5000 раз в сутки для одного сервиса
        """

        params = locals()
        method_name = self._base_method + 'save'
        return self._call(method_name, params)

    def search(self, q: str = None, sort: str = None, hd: str = None, adult: str = None, filters: str = None,
               search_own: str = None, offset: str = None, longer: str = None, shorter: str = None, count: str = None,
               extended: str = None) -> dict:
        """
        Возвращает список видеозаписей в соответствии с заданным критерием поиска.

        :param q: строка поискового запроса. Например, The Beatles.
        :param sort: сортировка результатов. Возможные значения:   0 — по дате добавления видеозаписи;  1 — по
            длительности;  2 — по релевантности.
        :param hd: если не равен нулю, то поиск производится только по видеозаписям высокого качества.
        :param adult: фильтр «Безопасный поиск». Возможные значения:   1 — выключен;  0 — включен.
        :param filters: список критериев, по которым требуется отфильтровать видео:   mp4 — искать только видео в
            формате mp4 (воспроизводимое на iOS);  youtube — возвращать только youtube видео;  vimeo — возвращать
            только vimeo видео;  short — возвращать только короткие видеозаписи;  long — возвращать только длинные
            видеозаписи.
        :param search_own: 1 — искать по видеозаписям пользователя, 0 — не искать по видеозаписям пользователя.
            По умолчанию: 0.
        :param offset: смещение относительно первой найденной видеозаписи для выборки определенного подмножества.
        :param longer: количество секунд, видеозаписи длиннее которого необходимо вернуть.
        :param shorter: количество секунд, видеозаписи короче которого необходимо вернуть.
        :param count: количество возвращаемых видеозаписей. Обратите внимание — даже при использовании параметра
            offset для получения информации доступны только первые 1000 результатов.
        :param extended: 1 — в ответе будут возвращены дополнительные поля profiles и groups, содержащие
            информацию о пользователях и сообществах. По умолчанию: 0.

        :return: После успешного выполнения возвращает объект, содержащий число результатов в поле count и массив
            объектов видеозаписей в поле items.  Если был задан параметр extended=1, возвращает число результатов в
            поле count, отдельно массив объектов видеозаписей в поле items, пользователей в поле profiles и сообществ
            в поле groups.  Если в Вашем приложении используется прямая авторизация, возвращается дополнительное поле
            files, содержащее ссылку на файл с видео (если ролик размещен на сервере ВКонтакте) или ссылку на внешний
            ресурс (если ролик встроен с какого-либо видеохостинга)
        """

        params = locals()
        method_name = self._base_method + 'search'
        return self._call(method_name, params)



from .Account import *
from .Ads import *
from .AppWidgets import *
from .Apps import *
from .Auth import *
from .Board import *
from .Database import *
from .Docs import *
from .DownloadedGames import *
from .Fave import *
from .Friends import *
from .Gifts import *
from .Groups import *
from .LeadForms import *
from .Leads import *
from Likes import *
from .Market import *
from .Newsfeed import *
from .Notes import *
from .Notifications import *
from .Orders import *
from .Pages import *
from .Photos import *
from .Podcasts import *
from .Polls import *
from .PrettyCards import *
from .Search import *
from .Secure import *
from .Stats import *
from .Status import *
from .Storage import *
from .Stories import *
from .Streaming import *
from .Users import *
from .Utils import *
from Video import *
from .Wall import *
from .Widgets import *

from .Messages import *
from .Execute import *
